/*
 * GXMame
 *
 * 2002, Stphane Pontier <shadow_walker@users.sourceforge.net>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <sys/types.h>
#include <sys/stat.h>
#include <unistd.h>
#include <string.h>
#include <stdio.h>

#include <gdk/gdkkeysyms.h>
#include <gtk/gtk.h>

#include "gxmame.h"
#include "callbacks.h"
#include "interface.h"
#include "support.h"
#include "directories.h"

GtkWidget *directories_selection;
GList *roms_path_list = NULL;
GList *samples_path_list = NULL;
gpointer the_current_entry;

GtkWidget *roms_path_entry;
GtkWidget *roms_path_gtk_list;

GtkWidget *samples_path_entry;
GtkWidget *samples_path_gtk_list;

GtkWidget *snapshot_path_entry;
GtkWidget *artwork_path_entry;
GtkWidget *input_path_entry;
GtkWidget *cheatfile_path_entry;
GtkWidget *hiscorefile_path_entry;
GtkWidget *historyfile_path_entry;
GtkWidget *mameinfofile_path_entry;
GtkWidget *hiscore_path_entry;
GtkWidget *xmame_exe_path_entry;
GtkWidget *common_path_entry;
GtkWidget *current_sample_list_item;
GtkWidget *current_rom_list_item;

GtkWidget *directories_selection_apply_button;

GtkWidget*
create_folderselection (gpointer current_entry,gboolean directory_selection_box)
{
	GtkWidget *folderselection;
	GtkWidget *ok_button_folderselection;
	GtkWidget *cancel_button_folderselection;
	gchar *temp_text;

	the_current_entry = current_entry;
	if (directory_selection_box)
		folderselection = gtk_file_selection_new (_("Browse for Folder"));
	else
		folderselection = gtk_file_selection_new (_("Browse for File"));
	gtk_object_set_data (GTK_OBJECT (folderselection), "folderselection", folderselection);
	gtk_container_set_border_width (GTK_CONTAINER (folderselection), 10);
	GTK_WINDOW (folderselection)->type = GTK_WINDOW_DIALOG;
	gtk_window_set_position (GTK_WINDOW (folderselection), GTK_WIN_POS_MOUSE);
	gtk_window_set_modal (GTK_WINDOW (folderselection), TRUE);
	gtk_window_set_policy (GTK_WINDOW (folderselection), FALSE, FALSE, FALSE);
	gtk_window_set_transient_for(GTK_WINDOW(folderselection),GTK_WINDOW(directories_selection));
	gtk_file_selection_hide_fileop_buttons (GTK_FILE_SELECTION (folderselection));
	if (directory_selection_box)
		gtk_widget_hide (GTK_WIDGET (GTK_FILE_SELECTION(folderselection)->file_list)->parent);
	
	temp_text = gtk_editable_get_chars (GTK_EDITABLE(current_entry),0,-1);
	gtk_file_selection_set_filename(GTK_FILE_SELECTION(folderselection),temp_text);
	g_free(temp_text);
	ok_button_folderselection = GTK_FILE_SELECTION (folderselection)->ok_button;
	gtk_object_set_data (GTK_OBJECT (folderselection), "ok_button_folderselection", ok_button_folderselection);
	gtk_widget_show (ok_button_folderselection);
	GTK_WIDGET_SET_FLAGS (ok_button_folderselection, GTK_CAN_DEFAULT);

	cancel_button_folderselection = GTK_FILE_SELECTION (folderselection)->cancel_button;
	gtk_object_set_data (GTK_OBJECT (folderselection), "cancel_button_folderselection", cancel_button_folderselection);
	gtk_widget_show (cancel_button_folderselection);
	GTK_WIDGET_SET_FLAGS (cancel_button_folderselection, GTK_CAN_DEFAULT);

	gtk_signal_connect (GTK_OBJECT (ok_button_folderselection), "clicked",
			    GTK_SIGNAL_FUNC (on_ok_button_folderselection_clicked),
			    (gpointer) folderselection);
	gtk_signal_connect (GTK_OBJECT (cancel_button_folderselection), "clicked",
			    GTK_SIGNAL_FUNC (on_cancel_button_folderselection_clicked),
			    (gpointer) folderselection);
	
	gtk_widget_show(folderselection);
	if (directory_selection_box)
		set_icon(folderselection,_("Folder Selection"));
	else
		set_icon(folderselection,_("File Selection"));
	return folderselection;
}
GtkWidget*
create_directories_selection (void)
{
	GtkWidget *vbox4;
	GtkWidget *frame1;
	GtkWidget *table1;
	GtkWidget *label3;
	GtkWidget *label4;
	GtkWidget *xmame_exe_browse_button;
	GtkWidget *common_browse_button;
	GtkWidget *label5;
	GtkWidget *frame2;
	GtkWidget *table2;
	GtkWidget *label6;
	GtkWidget *vbox5;
	GtkWidget *scrolledwindow4;
	GtkWidget *viewport4;
	GtkWidget *label7;
	GtkWidget *vbox6;
	GtkWidget *scrolledwindow5;
	GtkWidget *label9;
	GtkWidget *vbox7;
	GtkWidget *samples_browse_button;
	GtkWidget *samples_add_button;
	GtkWidget *samples_remove_button;
	GtkWidget *vbox8;
	GtkWidget *viewport5;
	GtkWidget *roms_browse_button;
	GtkWidget *roms_add_button;
	GtkWidget *roms_remove_button;
	GtkWidget *label10;
	GtkWidget *snapshots_browse_button;
	GtkWidget *label11;
	GtkWidget *hbox2;
	GtkWidget *artwork_browse_button;
	GtkWidget *label12;
	GtkWidget *hiscore_browse_button;
	GtkWidget *label13;
	GtkWidget *input_browse_button;
	GtkWidget *hseparator1;
	GtkWidget *label20;
	GtkWidget *cheatfile_browse_button;
	GtkWidget *label21;
	GtkWidget *hiscorefile_browse_button;
	GtkWidget *label22;
	GtkWidget *historyfile_browse_button;
	GtkWidget *label23;
	GtkWidget *mameinfofile_browse_button;
	GtkWidget *directories_selection_cancel_button;
	GtkWidget *directories_selection_ok_button;
	gint i;

	if (roms_path_list == NULL)
	{
		for (i=0;gui_prefs.RomPath[i]!=NULL;i++)
		{
			roms_path_list = g_list_append (roms_path_list,gtk_list_item_new_with_label(gui_prefs.RomPath[i]));
		}
	}
	if (samples_path_list == NULL)
	{
		for (i=0;gui_prefs.SamplePath[i]!=NULL;i++)
		{
			samples_path_list = g_list_append (samples_path_list,gtk_list_item_new_with_label(gui_prefs.SamplePath[i]));
		}
	}
	

	directories_selection = gtk_window_new (GTK_WINDOW_DIALOG);
	gtk_object_set_data (GTK_OBJECT (directories_selection), "directories_selection", directories_selection);
	gtk_window_set_title (GTK_WINDOW (directories_selection), _("Directories Selection"));
	gtk_window_set_transient_for(GTK_WINDOW(directories_selection),GTK_WINDOW(MainWindow));

	vbox4 = gtk_vbox_new (FALSE, 0);
	gtk_widget_ref (vbox4);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "vbox4", vbox4,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (vbox4);
	gtk_container_add (GTK_CONTAINER (directories_selection), vbox4);

	frame1 = gtk_frame_new (_("GXMame Directories"));
	gtk_widget_ref (frame1);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "frame1", frame1,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (frame1);
	gtk_box_pack_start (GTK_BOX (vbox4), frame1, TRUE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (frame1), 5);

	table1 = gtk_table_new (3, 3, FALSE);
	gtk_widget_ref (table1);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "table1", table1,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (table1);
	gtk_container_add (GTK_CONTAINER (frame1), table1);
	gtk_container_set_border_width (GTK_CONTAINER (table1), 2);
	gtk_table_set_row_spacings (GTK_TABLE (table1), 5);
	gtk_table_set_col_spacings (GTK_TABLE (table1), 5);

	label3 = gtk_label_new (_("Path to xmame and various config files"));
	gtk_widget_ref (label3);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "label3", label3,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label3);
	gtk_table_attach (GTK_TABLE (table1), label3, 1, 2, 0, 1,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label3), 0, 0.5);

	label4 = gtk_label_new (_("Xmame executable Path"));
	gtk_widget_ref (label4);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "label4", label4,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label4);
	gtk_table_attach (GTK_TABLE (table1), label4, 0, 1, 1, 2,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label4), 0, 0.5);

	xmame_exe_path_entry = gtk_entry_new ();
	gtk_widget_ref (xmame_exe_path_entry);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "xmame_exe_path_entry", xmame_exe_path_entry,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_entry_set_text (GTK_ENTRY (xmame_exe_path_entry), mame_executable);
	gtk_widget_show (xmame_exe_path_entry);
	gtk_table_attach (GTK_TABLE (table1), xmame_exe_path_entry, 1, 2, 1, 2,
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	xmame_exe_browse_button = gtk_button_new_with_label (_("Browse..."));
	gtk_widget_ref (xmame_exe_browse_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "xmame_exe_browse_button", xmame_exe_browse_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (xmame_exe_browse_button);
	gtk_table_attach (GTK_TABLE (table1), xmame_exe_browse_button, 2, 3, 1, 2,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	common_browse_button = gtk_button_new_with_label (_("Browse..."));
	gtk_widget_ref (common_browse_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "common_browse_button", common_browse_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (common_browse_button);
	gtk_table_attach (GTK_TABLE (table1), common_browse_button, 2, 3, 2, 3,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	common_path_entry = gtk_entry_new ();
	gtk_widget_ref (common_path_entry);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "common_path_entry", common_path_entry,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_entry_set_text (GTK_ENTRY (common_path_entry), gui_prefs.catverDirectory);
	gtk_widget_show (common_path_entry);
	gtk_table_attach (GTK_TABLE (table1), common_path_entry, 1, 2, 2, 3,
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	label5 = gtk_label_new (_("catver path"));
	gtk_widget_ref (label5);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "label5", label5,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label5);
	gtk_table_attach (GTK_TABLE (table1), label5, 0, 1, 2, 3,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label5), 0, 0.5);

	frame2 = gtk_frame_new (_("X-Mame Directories"));
	gtk_widget_ref (frame2);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "frame2", frame2,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (frame2);
	gtk_box_pack_start (GTK_BOX (vbox4), frame2, TRUE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (frame2), 5);

	table2 = gtk_table_new (4, 3, FALSE);
	gtk_widget_ref (table2);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "table2", table2,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (table2);
	gtk_container_add (GTK_CONTAINER (frame2), table2);
	gtk_container_set_border_width (GTK_CONTAINER (table2), 2);
	gtk_table_set_row_spacings (GTK_TABLE (table2), 5);
	gtk_table_set_col_spacings (GTK_TABLE (table2), 5);

	label6 = gtk_label_new (_("path to files needed for x-mame"));
	gtk_widget_ref (label6);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "label6", label6,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label6);
	gtk_table_attach (GTK_TABLE (table2), label6, 1, 2, 0, 1,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label6), 0, 0.5);

/* Romspath directories*/
	label7 = gtk_label_new (_("Roms Path"));
	gtk_widget_ref (label7);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "label7", label7,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label7);
	gtk_table_attach (GTK_TABLE (table2), label7, 0, 1, 1, 2,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (GTK_FILL), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label7), 0, 0);
	gtk_misc_set_padding (GTK_MISC (label7), 0, 7);

	vbox5 = gtk_vbox_new (FALSE, 0);
	gtk_widget_ref (vbox5);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "vbox5", vbox5,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (vbox5);
	gtk_table_attach (GTK_TABLE (table2), vbox5, 1, 2, 1, 2,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (GTK_FILL), 0, 0);

	roms_path_entry = gtk_entry_new ();
	gtk_widget_ref (roms_path_entry);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "roms_path_entry", roms_path_entry,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (roms_path_entry);
	gtk_box_pack_start (GTK_BOX (vbox5), roms_path_entry, FALSE, FALSE, 0);

	scrolledwindow4 = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_ref (scrolledwindow4);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "scrolledwindow4", scrolledwindow4,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (scrolledwindow4);
	gtk_box_pack_start (GTK_BOX (vbox5), scrolledwindow4, TRUE, TRUE, 0);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow4), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	viewport4 = gtk_viewport_new (NULL, NULL);
	gtk_widget_ref (viewport4);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "viewport4", viewport4,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (viewport4);
	gtk_container_add (GTK_CONTAINER (scrolledwindow4), viewport4);

	roms_path_gtk_list = gtk_list_new ();
	gtk_widget_ref (roms_path_gtk_list);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "roms_path_gtk_list", roms_path_gtk_list,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_list_append_items(GTK_LIST(roms_path_gtk_list) ,roms_path_list);
	gtk_widget_show_all (roms_path_gtk_list);
	gtk_container_add (GTK_CONTAINER (viewport4), roms_path_gtk_list);

	vbox8 = gtk_vbox_new (FALSE, 0);
	gtk_widget_ref (vbox8);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "vbox8", vbox8,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (vbox8);
	gtk_table_attach (GTK_TABLE (table2), vbox8, 2, 3, 1, 2,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (GTK_SHRINK | GTK_FILL), 0, 0);

	roms_browse_button = gtk_button_new_with_label (_("Browse..."));
	gtk_widget_ref (roms_browse_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "roms_browse_button", roms_browse_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (roms_browse_button);
	gtk_box_pack_start (GTK_BOX (vbox8), roms_browse_button, FALSE, FALSE, 2);

	roms_add_button = gtk_button_new_with_label (_("Add"));
	gtk_widget_ref (roms_add_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "roms_add_button", roms_add_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (roms_add_button);
	gtk_box_pack_start (GTK_BOX (vbox8), roms_add_button, FALSE, FALSE, 2);

	roms_remove_button = gtk_button_new_with_label (_("Remove"));
	gtk_widget_ref (roms_remove_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "roms_remove_button", roms_remove_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (roms_remove_button);
	gtk_box_pack_start (GTK_BOX (vbox8), roms_remove_button, FALSE, FALSE, 2);

/* samplespath directories*/
	label10 = gtk_label_new (_("Samples Path"));
	gtk_widget_ref (label10);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "label10", label10,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label10);
	gtk_table_attach (GTK_TABLE (table2), label10, 0, 1, 2, 3,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (GTK_FILL), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label10), 0, 0);

	vbox6 = gtk_vbox_new (FALSE, 0);
	gtk_widget_ref (vbox6);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "vbox6", vbox6,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (vbox6);
	gtk_table_attach (GTK_TABLE (table2), vbox6, 1, 2, 2, 3,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (GTK_FILL), 0, 0);

	samples_path_entry = gtk_entry_new ();
	gtk_widget_ref (samples_path_entry);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "samples_path_entry", samples_path_entry,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (samples_path_entry);
	gtk_box_pack_start (GTK_BOX (vbox6), samples_path_entry, FALSE, FALSE, 0);

	scrolledwindow5 = gtk_scrolled_window_new (NULL, NULL);
	gtk_widget_ref (scrolledwindow5);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "scrolledwindow5", scrolledwindow5,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (scrolledwindow5);
	gtk_box_pack_start (GTK_BOX (vbox6), scrolledwindow5, TRUE, TRUE, 0);
	gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolledwindow5), GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);

	viewport5 = gtk_viewport_new (NULL, NULL);
	gtk_widget_ref (viewport5);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "viewport5", viewport5,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (viewport5);
	gtk_container_add (GTK_CONTAINER (scrolledwindow5), viewport5);

	samples_path_gtk_list = gtk_list_new ();
	gtk_widget_ref (samples_path_gtk_list);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "samples_path_gtk_list", samples_path_gtk_list,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_list_append_items(GTK_LIST(samples_path_gtk_list) ,samples_path_list);
	gtk_widget_show_all (samples_path_gtk_list);
	gtk_container_add (GTK_CONTAINER (viewport5), samples_path_gtk_list);

	vbox7 = gtk_vbox_new (FALSE, 0);
	gtk_widget_ref (vbox7);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "vbox7", vbox7,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (vbox7);
	gtk_table_attach (GTK_TABLE (table2), vbox7, 2, 3, 2, 3,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (GTK_SHRINK | GTK_FILL), 0, 0);

	samples_browse_button = gtk_button_new_with_label (_("Browse..."));
	gtk_widget_ref (samples_browse_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "samples_browse_button", samples_browse_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (samples_browse_button);
	gtk_box_pack_start (GTK_BOX (vbox7), samples_browse_button, FALSE, FALSE, 2);

	samples_add_button = gtk_button_new_with_label (_("Add"));
	gtk_widget_ref (samples_add_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "samples_add_button", samples_add_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (samples_add_button);
	gtk_box_pack_start (GTK_BOX (vbox7), samples_add_button, FALSE, FALSE, 2);

	samples_remove_button = gtk_button_new_with_label (_("Remove"));
	gtk_widget_ref (samples_remove_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "samples_remove_button", samples_remove_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (samples_remove_button);
	gtk_box_pack_start (GTK_BOX (vbox7), samples_remove_button, FALSE, FALSE, 2);

/* Snapshot directory */
	label9 = gtk_label_new (_("Snapshots Path"));
	gtk_widget_ref (label9);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "label9", label9,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label9);
	gtk_table_attach (GTK_TABLE (table2), label9, 0, 1, 3, 4,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label9), 0, 0.5);

	snapshot_path_entry = gtk_entry_new ();
	gtk_widget_ref (snapshot_path_entry);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "snapshot_path_entry", snapshot_path_entry,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_entry_set_text (GTK_ENTRY (snapshot_path_entry), gui_prefs.SnapshotDirectory);
	gtk_widget_show (snapshot_path_entry);
	gtk_table_attach (GTK_TABLE (table2), snapshot_path_entry, 1, 2, 3, 4,
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	snapshots_browse_button = gtk_button_new_with_label (_("Browse..."));
	gtk_widget_ref (snapshots_browse_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "snapshots_browse_button", snapshots_browse_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (snapshots_browse_button);
	gtk_table_attach (GTK_TABLE (table2), snapshots_browse_button, 2, 3, 3, 4,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	
/* Artwork directory */
	label11 = gtk_label_new (_("Artwork Path"));
	gtk_widget_ref (label11);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "label11", label11,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label11);
	gtk_table_attach (GTK_TABLE (table2), label11, 0, 1, 4, 5,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label11), 0, 0.5);

	artwork_path_entry = gtk_entry_new ();
	gtk_widget_ref (artwork_path_entry);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "artwork_path_entry", artwork_path_entry,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_entry_set_text (GTK_ENTRY (artwork_path_entry), gui_prefs.ArtworkDirectory);
	gtk_widget_show (artwork_path_entry);
	gtk_table_attach (GTK_TABLE (table2), artwork_path_entry, 1, 2, 4, 5,
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	artwork_browse_button = gtk_button_new_with_label (_("Browse..."));
	gtk_widget_ref (artwork_browse_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "artwork_browse_button", artwork_browse_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (artwork_browse_button);
	gtk_table_attach (GTK_TABLE (table2), artwork_browse_button, 2, 3, 4, 5,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	
/* hiscores directory */
	label12 = gtk_label_new (_("High Score Path"));
	gtk_widget_ref (label12);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "label12", label12,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label12);
	gtk_table_attach (GTK_TABLE (table2), label12, 0, 1, 5, 6,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label12), 0, 0.5);

	hiscore_path_entry = gtk_entry_new ();
	gtk_widget_ref (hiscore_path_entry);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "hiscore_path_entry", hiscore_path_entry,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_entry_set_text (GTK_ENTRY (hiscore_path_entry), gui_prefs.HiscoreDirectory);
	gtk_widget_show (hiscore_path_entry);
	gtk_table_attach (GTK_TABLE (table2), hiscore_path_entry, 1, 2, 5, 6,
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	hiscore_browse_button = gtk_button_new_with_label (_("Browse..."));
	gtk_widget_ref (hiscore_browse_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "hiscore_browse_button", hiscore_browse_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (hiscore_browse_button);
	gtk_table_attach (GTK_TABLE (table2), hiscore_browse_button, 2, 3, 5, 6,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	
/* input directory */
	label13 = gtk_label_new (_("Recorded games Path"));
	gtk_widget_ref (label13);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "label13", label13,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label13);
	gtk_table_attach (GTK_TABLE (table2), label13, 0, 1, 6, 7,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label13), 0, 0.5);

	input_path_entry = gtk_entry_new ();
	gtk_widget_ref (input_path_entry);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "input_path_entry", input_path_entry,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_entry_set_text (GTK_ENTRY (input_path_entry), gui_prefs.InputDirectory);
	gtk_widget_show (input_path_entry);
	gtk_table_attach (GTK_TABLE (table2), input_path_entry, 1, 2, 6, 7,
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	input_browse_button = gtk_button_new_with_label (_("Browse..."));
	gtk_widget_ref (input_browse_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "input_browse_button", input_browse_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (input_browse_button);
	gtk_table_attach (GTK_TABLE (table2), input_browse_button, 2, 3, 6, 7,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	
	hseparator1 = gtk_hseparator_new();
	gtk_widget_show(hseparator1);
	gtk_table_attach (GTK_TABLE (table2), hseparator1, 0, 3, 7, 8,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	
/* Cheat File */
	label20 = gtk_label_new (_("Cheat File"));
	gtk_widget_ref (label20);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "label20", label20,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label20);
	gtk_table_attach (GTK_TABLE (table2), label20, 0, 1, 8, 9,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label20), 0, 0.5);

	cheatfile_path_entry = gtk_entry_new ();
	gtk_widget_ref (cheatfile_path_entry);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "cheatfile_path_entry", cheatfile_path_entry,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_entry_set_text (GTK_ENTRY (cheatfile_path_entry), gui_prefs.CheatFile);
	gtk_widget_show (cheatfile_path_entry);
	gtk_table_attach (GTK_TABLE (table2), cheatfile_path_entry, 1, 2, 8, 9,
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	cheatfile_browse_button = gtk_button_new_with_label (_("Browse..."));
	gtk_widget_ref (cheatfile_browse_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "cheatfile_browse_button", cheatfile_browse_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (cheatfile_browse_button);
	gtk_table_attach (GTK_TABLE (table2), cheatfile_browse_button, 2, 3, 8, 9,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

/* hiscorefile File */
	label21 = gtk_label_new (_("HiScore File"));
	gtk_widget_ref (label21);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "label21", label21,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label21);
	gtk_table_attach (GTK_TABLE (table2), label21, 0, 1, 9, 10,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label21), 0, 0.5);

	hiscorefile_path_entry = gtk_entry_new ();
	gtk_widget_ref (hiscorefile_path_entry);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "hiscorefile_path_entry", hiscorefile_path_entry,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_entry_set_text (GTK_ENTRY (hiscorefile_path_entry), gui_prefs.HiscoreFile);
	gtk_widget_show (hiscorefile_path_entry);
	gtk_table_attach (GTK_TABLE (table2), hiscorefile_path_entry, 1, 2, 9, 10,
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	hiscorefile_browse_button = gtk_button_new_with_label (_("Browse..."));
	gtk_widget_ref (hiscorefile_browse_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "hiscorefile_browse_button", hiscorefile_browse_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (hiscorefile_browse_button);
	gtk_table_attach (GTK_TABLE (table2), hiscorefile_browse_button, 2, 3, 9, 10,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

/* historyfile File */
	label22 = gtk_label_new (_("History File"));
	gtk_widget_ref (label22);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "label22", label22,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label22);
	gtk_table_attach (GTK_TABLE (table2), label22, 0, 1, 10, 11,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label22), 0, 0.5);

	historyfile_path_entry = gtk_entry_new ();
	gtk_widget_ref (historyfile_path_entry);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "historyfile_path_entry", historyfile_path_entry,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_entry_set_text (GTK_ENTRY (historyfile_path_entry), gui_prefs.HistoryFile);
	gtk_widget_show (historyfile_path_entry);
	gtk_table_attach (GTK_TABLE (table2), historyfile_path_entry, 1, 2, 10, 11,
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	historyfile_browse_button = gtk_button_new_with_label (_("Browse..."));
	gtk_widget_ref (historyfile_browse_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "historyfile_browse_button", historyfile_browse_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (historyfile_browse_button);
	gtk_table_attach (GTK_TABLE (table2), historyfile_browse_button, 2, 3, 10, 11,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

/* mameinfofile File */
	label23 = gtk_label_new (_("Mame Info File"));
	gtk_widget_ref (label23);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "label23", label23,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (label23);
	gtk_table_attach (GTK_TABLE (table2), label23, 0, 1, 11, 12,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);
	gtk_misc_set_alignment (GTK_MISC (label23), 0, 0.5);

	mameinfofile_path_entry = gtk_entry_new ();
	gtk_widget_ref (mameinfofile_path_entry);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "mameinfofile_path_entry", mameinfofile_path_entry,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_entry_set_text (GTK_ENTRY (mameinfofile_path_entry), gui_prefs.MameInfoFile);
	gtk_widget_show (mameinfofile_path_entry);
	gtk_table_attach (GTK_TABLE (table2), mameinfofile_path_entry, 1, 2, 11, 12,
			  (GtkAttachOptions) (GTK_EXPAND | GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	mameinfofile_browse_button = gtk_button_new_with_label (_("Browse..."));
	gtk_widget_ref (mameinfofile_browse_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "mameinfofile_browse_button", mameinfofile_browse_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (mameinfofile_browse_button);
	gtk_table_attach (GTK_TABLE (table2), mameinfofile_browse_button, 2, 3, 11, 12,
			  (GtkAttachOptions) (GTK_FILL),
			  (GtkAttachOptions) (0), 0, 0);

	
		

	hbox2 = gtk_hbox_new (FALSE, 5);
	gtk_widget_ref (hbox2);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "hbox2", hbox2,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (hbox2);
	gtk_box_pack_start (GTK_BOX (vbox4), hbox2, FALSE, TRUE, 0);
	gtk_container_set_border_width (GTK_CONTAINER (hbox2), 5);

	directories_selection_cancel_button = gtk_button_new_with_label (_("Cancel"));
	gtk_widget_ref (directories_selection_cancel_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "directories_selection_cancel_button", directories_selection_cancel_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (directories_selection_cancel_button);
	gtk_box_pack_end (GTK_BOX (hbox2), directories_selection_cancel_button, FALSE, FALSE, 0);
	gtk_widget_set_usize (directories_selection_cancel_button, 50, -2);

	directories_selection_apply_button = gtk_button_new_with_label (_("Apply"));
	gtk_widget_ref (directories_selection_apply_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "directories_selection_apply_button", directories_selection_apply_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (directories_selection_apply_button);
	gtk_box_pack_end (GTK_BOX (hbox2), directories_selection_apply_button, FALSE, FALSE, 0);
	gtk_widget_set_usize (directories_selection_apply_button, 50, -2);

	gtk_widget_set_sensitive(directories_selection_apply_button, FALSE);
	
	directories_selection_ok_button = gtk_button_new_with_label (_("Ok"));
	gtk_widget_ref (directories_selection_ok_button);
	gtk_object_set_data_full (GTK_OBJECT (directories_selection), "directories_selection_ok_button", directories_selection_ok_button,
				  (GtkDestroyNotify) gtk_widget_unref);
	gtk_widget_show (directories_selection_ok_button);
	gtk_box_pack_end (GTK_BOX (hbox2), directories_selection_ok_button, FALSE, FALSE, 0);
	gtk_widget_set_usize (directories_selection_ok_button, 50, -2);

	gtk_signal_connect (GTK_OBJECT (xmame_exe_browse_button), "clicked",
			    GTK_SIGNAL_FUNC (on_file_browse_button_clicked),
			    xmame_exe_path_entry);
	gtk_signal_connect (GTK_OBJECT (common_browse_button), "clicked",
			    GTK_SIGNAL_FUNC (on_dir_browse_button_clicked),
			    common_path_entry);
	gtk_signal_connect (GTK_OBJECT (samples_browse_button), "clicked",
			    GTK_SIGNAL_FUNC (on_dir_browse_button_clicked),
			    samples_path_entry);
	gtk_signal_connect (GTK_OBJECT (samples_add_button), "clicked",
			    GTK_SIGNAL_FUNC (on_samples_add_button_clicked),
			    NULL);
	gtk_signal_connect (GTK_OBJECT (samples_remove_button), "clicked",
			    GTK_SIGNAL_FUNC (on_samples_remove_button_clicked),
			    NULL);
	gtk_signal_connect (GTK_OBJECT (roms_browse_button), "clicked",
			    GTK_SIGNAL_FUNC (on_dir_browse_button_clicked),
			    roms_path_entry);
	gtk_signal_connect (GTK_OBJECT (roms_add_button), "clicked",
			    GTK_SIGNAL_FUNC (on_roms_add_button_clicked),
			    NULL);
	gtk_signal_connect (GTK_OBJECT (roms_remove_button), "clicked",
			    GTK_SIGNAL_FUNC (on_roms_remove_button_clicked),
			    NULL);
	gtk_signal_connect (GTK_OBJECT (snapshots_browse_button), "clicked",
			    GTK_SIGNAL_FUNC (on_dir_browse_button_clicked),
			    snapshot_path_entry);
	gtk_signal_connect (GTK_OBJECT (artwork_browse_button), "clicked",
			    GTK_SIGNAL_FUNC (on_dir_browse_button_clicked),
			    artwork_path_entry);
	gtk_signal_connect (GTK_OBJECT (input_browse_button), "clicked",
			    GTK_SIGNAL_FUNC (on_dir_browse_button_clicked),
			    input_path_entry);
	gtk_signal_connect (GTK_OBJECT (hiscore_browse_button), "clicked",
			    GTK_SIGNAL_FUNC (on_dir_browse_button_clicked),
			    hiscore_path_entry);

	gtk_signal_connect (GTK_OBJECT (cheatfile_browse_button), "clicked",
			    GTK_SIGNAL_FUNC (on_file_browse_button_clicked),
			    cheatfile_path_entry);
	gtk_signal_connect (GTK_OBJECT (hiscorefile_browse_button), "clicked",
			    GTK_SIGNAL_FUNC (on_file_browse_button_clicked),
			    hiscorefile_path_entry);
	gtk_signal_connect (GTK_OBJECT (historyfile_browse_button), "clicked",
			    GTK_SIGNAL_FUNC (on_file_browse_button_clicked),
			    historyfile_path_entry);
	gtk_signal_connect (GTK_OBJECT (mameinfofile_browse_button), "clicked",
			    GTK_SIGNAL_FUNC (on_file_browse_button_clicked),
			    mameinfofile_path_entry);
	
	gtk_signal_connect (GTK_OBJECT (directories_selection_cancel_button), "clicked",
			    GTK_SIGNAL_FUNC (directories_selection_close_window),
			    directories_selection);
	gtk_signal_connect (GTK_OBJECT (directories_selection_apply_button), "clicked",
			    GTK_SIGNAL_FUNC (directories_selection_save_changes),
			    directories_selection);
	gtk_signal_connect (GTK_OBJECT (directories_selection_ok_button), "clicked",
			    GTK_SIGNAL_FUNC (directories_selection_save_and_close),
			    directories_selection);
	gtk_signal_connect (GTK_OBJECT (samples_path_gtk_list), "select-child",
			    GTK_SIGNAL_FUNC (on_gtk_list_sample_directory_selected),
			    samples_path_entry);
	gtk_signal_connect (GTK_OBJECT (roms_path_gtk_list), "select-child",
			    GTK_SIGNAL_FUNC (on_gtk_list_rom_directory_selected),
			    roms_path_entry);
	gtk_signal_connect (GTK_OBJECT (directories_selection), "delete-event",
			    GTK_SIGNAL_FUNC (directories_selection_close_window),
			    directories_selection);
	
	gtk_signal_connect_object (GTK_OBJECT (mameinfofile_path_entry), "changed",
			    GTK_SIGNAL_FUNC (gtk_widget_set_sensitive),
			    GTK_OBJECT(directories_selection_apply_button));
	gtk_signal_connect_object (GTK_OBJECT (historyfile_path_entry), "changed",
			    GTK_SIGNAL_FUNC (gtk_widget_set_sensitive),
			    GTK_OBJECT(directories_selection_apply_button));
	gtk_signal_connect_object (GTK_OBJECT (hiscorefile_path_entry), "changed",
			    GTK_SIGNAL_FUNC (gtk_widget_set_sensitive),
			    GTK_OBJECT(directories_selection_apply_button));
	gtk_signal_connect_object (GTK_OBJECT (cheatfile_path_entry), "changed",
			    GTK_SIGNAL_FUNC (gtk_widget_set_sensitive),
			    GTK_OBJECT(directories_selection_apply_button));
	gtk_signal_connect_object (GTK_OBJECT (input_path_entry), "changed",
			    GTK_SIGNAL_FUNC (gtk_widget_set_sensitive),
			    GTK_OBJECT(directories_selection_apply_button));
	gtk_signal_connect_object (GTK_OBJECT (hiscore_path_entry), "changed",
			    GTK_SIGNAL_FUNC (gtk_widget_set_sensitive),
			    GTK_OBJECT(directories_selection_apply_button));
	gtk_signal_connect_object (GTK_OBJECT (artwork_path_entry), "changed",
			    GTK_SIGNAL_FUNC (gtk_widget_set_sensitive),
			    GTK_OBJECT(directories_selection_apply_button));
	gtk_signal_connect_object (GTK_OBJECT (snapshot_path_entry), "changed",
			    GTK_SIGNAL_FUNC (gtk_widget_set_sensitive),
			    GTK_OBJECT(directories_selection_apply_button));
	gtk_signal_connect_object (GTK_OBJECT (common_path_entry), "changed",
			    GTK_SIGNAL_FUNC (gtk_widget_set_sensitive),
			    GTK_OBJECT(directories_selection_apply_button));
	gtk_signal_connect_object (GTK_OBJECT (xmame_exe_path_entry), "changed",
			    GTK_SIGNAL_FUNC (gtk_widget_set_sensitive),
			    GTK_OBJECT(directories_selection_apply_button));

	
	return directories_selection;
}




void
on_dir_browse_button_clicked           (GtkButton       *button,
                                        gpointer         user_data)
{
	create_folderselection(user_data, TRUE);
}


void
on_file_browse_button_clicked         (GtkButton       *button,
                                        gpointer         user_data)
{
	create_folderselection(user_data, FALSE);
}


void
on_samples_add_button_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
	gchar *temp_text;
	GList *temp_list = NULL,*list_ptr = NULL; 
	temp_text = gtk_editable_get_chars (GTK_EDITABLE(samples_path_entry),0,-1);
	gxmame_debug("adding:%s to samples folders",temp_text);
	/* I need to copy the list otherwise, I can't add it again to the gtk list*/
	for(list_ptr = g_list_first(samples_path_list);list_ptr!=NULL;list_ptr=g_list_next(list_ptr))
	{
		temp_list = g_list_append(temp_list,list_ptr->data);
	}
	gtk_list_remove_items_no_unref(GTK_LIST(samples_path_gtk_list) ,samples_path_list);
	temp_list = g_list_append (temp_list,gtk_list_item_new_with_label(temp_text));
	gtk_list_append_items(GTK_LIST(samples_path_gtk_list) ,temp_list);
	gtk_widget_show_all (samples_path_gtk_list);
	g_free(temp_text);
	samples_path_list = temp_list;
	gtk_widget_set_sensitive(directories_selection_apply_button, TRUE);
}


void
on_samples_remove_button_clicked       (GtkButton       *button,
                                        gpointer         user_data)
{
	GList *temp_list = NULL,*list_ptr = NULL; 
	/* I need to copy the list otherwise, I can't add it again to the gtk list*/
	for(list_ptr = g_list_first(samples_path_list);list_ptr!=NULL;list_ptr=g_list_next(list_ptr))
	{
		temp_list = g_list_append(temp_list,list_ptr->data);
	}
	temp_list = g_list_remove(temp_list,current_sample_list_item);
	gtk_list_remove_items_no_unref(GTK_LIST(samples_path_gtk_list) ,samples_path_list);
	gtk_list_append_items(GTK_LIST(samples_path_gtk_list) ,temp_list);
	gtk_widget_show_all (samples_path_gtk_list);
	samples_path_list = temp_list;
	gtk_widget_set_sensitive(directories_selection_apply_button, TRUE);
}


void
on_roms_add_button_clicked             (GtkButton       *button,
                                        gpointer         user_data)
{
	gchar *temp_text;
	GList *temp_list = NULL,*list_ptr = NULL; 
	temp_text = gtk_editable_get_chars (GTK_EDITABLE(roms_path_entry),0,-1);
	gxmame_debug("adding:%s to roms folders",temp_text);
	/* I need to copy the list otherwise, I can't add it again to the gtk list*/
	for(list_ptr = g_list_first(roms_path_list);list_ptr!=NULL;list_ptr=g_list_next(list_ptr))
	{
		temp_list = g_list_append(temp_list,list_ptr->data);
	}
	gtk_list_remove_items_no_unref(GTK_LIST(roms_path_gtk_list) ,roms_path_list);
	temp_list = g_list_append (temp_list,gtk_list_item_new_with_label(temp_text));
	gtk_list_append_items(GTK_LIST(roms_path_gtk_list) ,temp_list);
	gtk_widget_show_all (roms_path_gtk_list);
	g_free(temp_text);
	roms_path_list = temp_list;
	gtk_widget_set_sensitive(directories_selection_apply_button, TRUE);
}


void
on_roms_remove_button_clicked          (GtkButton       *button,
                                        gpointer         user_data)
{
	GList *temp_list = NULL,*list_ptr = NULL; 
	/* I need to copy the list otherwise, I can't add it again to the gtk list*/
	for(list_ptr = g_list_first(roms_path_list);list_ptr!=NULL;list_ptr=g_list_next(list_ptr))
	{
		temp_list = g_list_append(temp_list,list_ptr->data);
	}
	temp_list = g_list_remove(temp_list,current_rom_list_item);
	gtk_list_remove_items_no_unref(GTK_LIST(roms_path_gtk_list) ,roms_path_list);
	gtk_list_append_items(GTK_LIST(roms_path_gtk_list) ,temp_list);
	gtk_widget_show_all (roms_path_gtk_list);
	roms_path_list = temp_list;
	gtk_widget_set_sensitive(directories_selection_apply_button, TRUE);
}


void
directories_selection_close_window
                                        (GtkButton       *button,
                                        gpointer         user_data)
{
	gtk_list_remove_items(GTK_LIST(roms_path_gtk_list) ,roms_path_list);
	gtk_list_remove_items(GTK_LIST(samples_path_gtk_list) ,samples_path_list);
	/* test if its called from the cancel button or if the window was destroyed*/
	if (GTK_IS_WIDGET(user_data))
		gtk_widget_destroy(user_data);
	else
		gtk_widget_destroy(GTK_WIDGET(button));
	roms_path_list = NULL;
	samples_path_list = NULL;
}


void
directories_selection_save_changes
                                        (GtkButton       *button,
                                        gpointer         user_data)
{
	GList *list_ptr;
	int size;
	/* free the main config paths*/
	g_free (gui_prefs.catverDirectory);
	g_free (mame_executable);
	g_free (gui_prefs.SnapshotDirectory);
	g_free (gui_prefs.ArtworkDirectory);
	g_free (gui_prefs.HiscoreDirectory);
	g_free (gui_prefs.InputDirectory);
	g_strfreev (gui_prefs.RomPath);
	g_strfreev (gui_prefs.SamplePath);
	g_free (gui_prefs.CheatFile);
	g_free (gui_prefs.HiscoreFile);
	g_free (gui_prefs.HistoryFile);
	g_free (gui_prefs.MameInfoFile);
	/* and replace them by the new ones*/
	gui_prefs.catverDirectory = gtk_editable_get_chars (GTK_EDITABLE(common_path_entry),0,-1);
	mame_executable = gtk_editable_get_chars (GTK_EDITABLE(xmame_exe_path_entry),0,-1);
	gui_prefs.SnapshotDirectory = gtk_editable_get_chars (GTK_EDITABLE(snapshot_path_entry),0,-1);
	gui_prefs.ArtworkDirectory = gtk_editable_get_chars (GTK_EDITABLE(artwork_path_entry),0,-1);
	gui_prefs.HiscoreDirectory = gtk_editable_get_chars (GTK_EDITABLE(hiscore_path_entry),0,-1);
	gui_prefs.InputDirectory = gtk_editable_get_chars (GTK_EDITABLE(input_path_entry),0,-1);
	gui_prefs.CheatFile = gtk_editable_get_chars (GTK_EDITABLE(cheatfile_path_entry),0,-1);
	gui_prefs.HiscoreFile = gtk_editable_get_chars (GTK_EDITABLE(hiscorefile_path_entry),0,-1);
	gui_prefs.HistoryFile = gtk_editable_get_chars (GTK_EDITABLE(historyfile_path_entry),0,-1);
	gui_prefs.MameInfoFile = gtk_editable_get_chars (GTK_EDITABLE(mameinfofile_path_entry),0,-1);
	
	size = g_list_length(samples_path_list)+1;
	gui_prefs.SamplePath = g_new(gchar *, size);
	for (size=0,list_ptr = g_list_first(samples_path_list);
	     (list_ptr != NULL);
	      list_ptr= g_list_next(list_ptr),size++)
	{
		gui_prefs.SamplePath[size] = g_strdup(GTK_LABEL(GTK_BIN(list_ptr->data)->child)->label);
	}
	gui_prefs.SamplePath[size]=NULL;
	
	size = g_list_length(roms_path_list)+1;
	gui_prefs.RomPath = g_new(gchar *, size);
	for (size=0,list_ptr = g_list_first(roms_path_list);
	     (list_ptr != NULL);
	      list_ptr= g_list_next(list_ptr),size++)
	{
		gui_prefs.RomPath[size] = g_strdup(GTK_LABEL(GTK_BIN(list_ptr->data)->child)->label);
	}
	gui_prefs.RomPath[size]=NULL;
	gtk_widget_set_sensitive(directories_selection_apply_button, FALSE);
}




void
directories_selection_save_and_close
                                        (GtkButton       *button,
                                        gpointer         user_data)
{
	directories_selection_save_changes(NULL,user_data);
	directories_selection_close_window(NULL,user_data);
}



void
on_ok_button_folderselection_clicked        (GtkButton       *button,
                                             gpointer         user_data)
{
	gxmame_debug("selected: {%s}",gtk_file_selection_get_filename (GTK_FILE_SELECTION(user_data)));
	gtk_entry_set_text (GTK_ENTRY (the_current_entry),gtk_file_selection_get_filename (GTK_FILE_SELECTION(user_data)));
	gtk_widget_destroy(user_data);
	gtk_widget_set_sensitive(directories_selection_apply_button, TRUE);
}


void
on_cancel_button_folderselection_clicked    (GtkButton       *button,
                                             gpointer         user_data)
{
	gtk_widget_destroy(user_data);
}

	
void
on_gtk_list_rom_directory_selected         (GtkList *list,
                                            GtkWidget *widget,
                                            gpointer user_data)
{
	current_rom_list_item = widget;
	gtk_entry_set_text (GTK_ENTRY (user_data), GTK_LABEL(GTK_BIN(widget)->child)->label);
}

void
on_gtk_list_sample_directory_selected      (GtkList *list,
                                            GtkWidget *widget,
                                            gpointer user_data)
{
	current_sample_list_item = widget;
	gtk_entry_set_text (GTK_ENTRY (user_data), GTK_LABEL(GTK_BIN(widget)->child)->label);
}

		
		
		
