/*
 * GXMame
 *
 * 2002, Stphane Pontier <shadow_walker@users.sourceforge.net>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <gtk/gtk.h>

#include <X11/Xlib.h>
#include <gdk/gdkx.h>
#include <sys/stat.h>

#include "interface.h"
#include "support.h"
#include "gxmame.h"
#include "io.h"
#include "mameio.h"
#include "progression_window.h"

#include <libintl.h>
#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#define BUFFER_SIZE 1000

void gxmame_init(void);


int main (int argc, char *argv[])
{
#ifdef ENABLE_NLS
	bindtextdomain (PACKAGE, PACKAGE_LOCALE_DIR);
	textdomain (PACKAGE);
#endif

	gtk_set_locale ();
	gtk_init (&argc, &argv);

	add_pixmap_directory ("/usr/share/pixmaps");
	add_pixmap_directory ("/usr/local/share/pixmaps");

	gxmame_init();

	init_gui();
	
	quick_check();

	/* got a segfault here if I'm trying to flush gtk event
	   and GUIPosX and GUIPosY are not Null */
//	if(!(gui_prefs.GUIPosX && gui_prefs.GUIPosY))
//		while(gtk_events_pending())
//			gtk_main_iteration();
	/*FIXME this is just to display the game position properly
	  when I got the strange 0.0 position bug*/
	create_gamelist_content();
	
	gtk_main ();
  return 0;
}

void gxmame_debug(const gchar * format, ...)
{
#ifdef DEBUG
	va_list args;
	va_start(args, format);
	g_logv(NULL, G_LOG_LEVEL_DEBUG, format, args);
	va_end(args);
#endif
}


void gxmame_init()
{
	GTimer *mytimer;
	gchar *filename, *dirname;
	struct stat filestatbuf;
/* should load pref here*/
	filename = g_malloc(50);
	dirname = g_malloc(50);
	
	mytimer=g_timer_new();
	g_timer_start(mytimer);
	g_snprintf(filename, 50, "%s/.gxmame",getenv("HOME"));
	if (stat(filename,&filestatbuf)<0) 
	{
		gxmame_debug ("no initial directory creating one");
		g_snprintf(dirname,50, "mkdir %s",filename);
		system(dirname);
		
	}
	g_free(filename);
	g_free(dirname);
	
	/* Load master config file*/
	if (!load_gxmamerc())
		g_message(_("gxmamerc not loaded, using default values"));
	
	get_mame_version(EXECUTABLE);
	
	if (!load_dirs_ini())
		g_message(_("dirs.ini not loaded, using default values"));
	if (!load_gxmame_ini())
		g_message(_("unable to load gxmame.ini, using default values"));

	if (!load_games_list())
	{
		g_message(_("gamelist not found, need to rebuild one"));
//		create_game_list_file();
//		load_games_list();
	}
	else
	{
		if (!load_games_ini())
			g_message(_("games.ini not loaded, using default values"));
		if (!load_catver_ini())
			g_message(_("catver not loaded, using default values"));
	}


	g_timer_stop(mytimer);
	g_message(_("time elapsed:%e"),g_timer_elapsed(mytimer,NULL));
	srand(g_timer_elapsed(mytimer,NULL));
	g_timer_destroy(mytimer);
	
		
	/*TEMP creation of a fixed columns list*/
	Columns = NULL;
	Columns = g_list_append(Columns, GINT_TO_POINTER(GAMENAME));
//	Columns = g_list_append(Columns, GINT_TO_POINTER(HAS_ROMS));
	Columns = g_list_append(Columns, GINT_TO_POINTER(HAS_SAMPLES));
	Columns = g_list_append(Columns, GINT_TO_POINTER(ROMNAME));
	Columns = g_list_append(Columns, GINT_TO_POINTER(VECTOR));
	Columns = g_list_append(Columns, GINT_TO_POINTER(CONTROL));
	Columns = g_list_append(Columns, GINT_TO_POINTER(TIMESPLAYED));
	Columns = g_list_append(Columns, GINT_TO_POINTER(MANU));
	Columns = g_list_append(Columns, GINT_TO_POINTER(YEAR));
	Columns = g_list_append(Columns, GINT_TO_POINTER(CLONE));
	Columns = g_list_append(Columns, GINT_TO_POINTER(DRIVER));
//	Columns = g_list_append(Columns, GINT_TO_POINTER(ROMOF));
//	Columns = g_list_append(Columns, GINT_TO_POINTER(MAMEVER));
//	Columns = g_list_append(Columns, GINT_TO_POINTER(CATEGORY));
	
	
}


gboolean game_filtered (RomEntry * rom)
{
	gchar *manufacturer1 = NULL,*manufacturer2 = NULL ,**manufacturer_fields = NULL;
/* comparing taxt and text */
	if (current_filter->type == DRIVER)
	{
		if ((current_filter->is && !g_strcasecmp(rom->driver,(gchar *)current_filter->value)) ||
		    (!current_filter->is && g_strcasecmp(rom->driver,(gchar *)current_filter->value))
		    ) {
			return TRUE;
		}
	} else if (current_filter->type == CLONE)
	{
		if ((current_filter->is && !g_strcasecmp(rom->cloneof,(gchar *)current_filter->value)) ||
		    (!current_filter->is && g_strcasecmp(rom->cloneof,(gchar *)current_filter->value))
		    ) {
			return TRUE;
		}
	} else if (current_filter->type == CONTROL)
	{
		if ((current_filter->is && !g_strcasecmp(rom->control,(gchar *)current_filter->value)) ||
		    (!current_filter->is && g_strcasecmp(rom->control,(gchar *)current_filter->value))
		    ) {
			return TRUE;
		}
	} else if (current_filter->type == MAMEVER)
	{
		if ((current_filter->is && !g_strcasecmp(rom->mame_ver_added,(gchar *)current_filter->value)) ||
		    (!current_filter->is && g_strcasecmp(rom->mame_ver_added,(gchar *)current_filter->value))
		    ) {
			return TRUE;
		}
	} else if (current_filter->type == CATEGORY)
	{
		if ((current_filter->is && !g_strcasecmp(rom->category,(gchar *)current_filter->value)) ||
		    (!current_filter->is && g_strcasecmp(rom->category,(gchar *)current_filter->value))
		    ) {
			return TRUE;
		}
/* comparing boolean and boolean */
	} else if (current_filter->type == FAVORITE)
	{
		if ((current_filter->is && rom->favourite)||
		    (!current_filter->is && !rom->favourite)
		    ) {
			return TRUE;
		}
	} else if (current_filter->type == VECTOR)
	{
		if ((current_filter->is && rom->vector)||
		    (!current_filter->is && !rom->vector)
		    ) {
			return TRUE;
		}
	} else if (current_filter->type == STATUS)
	{
		if ((current_filter->is && (rom->status))||
		    (!current_filter->is && !(rom->status))
		    ) {
			return TRUE;
		}
/* Comparing int and int */
	} else if (current_filter->type == HAS_ROMS)
	{
		if ((current_filter->is && (rom->has_roms == GPOINTER_TO_INT(current_filter->value)))  ||
		    (!current_filter->is && !(rom->has_roms == GPOINTER_TO_INT(current_filter->value)))
		    ) {
			return TRUE;
		}
	} else if (current_filter->type == TIMESPLAYED)
	{
		if ((current_filter->is && (rom->timesplayed == GPOINTER_TO_INT(current_filter->value))) ||
		    (!current_filter->is && !(rom->timesplayed == GPOINTER_TO_INT(current_filter->value)))
		    ) {
			return TRUE;
		}
	} else if (current_filter->type == CHANNELS)
	{
		if ((current_filter->is && (rom->channels == GPOINTER_TO_INT(current_filter->value))) ||
		    (!current_filter->is && !(rom->channels == GPOINTER_TO_INT(current_filter->value)))
		    ) {
			return TRUE;
		}
/* Comparing text and int */
	} else if (current_filter->type == YEAR)
	{
		if ((current_filter->is && (atoi(rom->year) == GPOINTER_TO_INT(current_filter->value))) ||
		    (!current_filter->is && !(atoi(rom->year) == GPOINTER_TO_INT(current_filter->value)))
		    ) {
			return TRUE;
		}
/* comparing parsed text and text */
	} else if (current_filter->type == MANU)
	{		
		manufacturer_fields = get_manufacturers(rom);
		/* we have now one or two clean manufacturer(s) we still need to differentiates sub companies*/
		if (manufacturer_fields[1] != NULL)
		{
			manufacturer1 = g_strdup(manufacturer_fields[0]);
			manufacturer2 = g_strdup(manufacturer_fields[1]);
			if ((current_filter->is && !g_strncasecmp(manufacturer1,(gchar *)current_filter->value,5)) ||
		    	    (!current_filter->is && g_strncasecmp(manufacturer1,(gchar *)current_filter->value,5)) ||
			    (current_filter->is && !g_strncasecmp(manufacturer2,(gchar *)current_filter->value,5)) ||
		    	    (!current_filter->is && g_strncasecmp(manufacturer2,(gchar *)current_filter->value,5))
		    		)
			{
				g_strfreev(manufacturer_fields);
				g_free(manufacturer1);g_free(manufacturer2);
				return TRUE;
			}
			g_free(manufacturer1);g_free(manufacturer2);
		}
		else
		{
			if ((current_filter->is && !g_strncasecmp(manufacturer_fields[0],(gchar *)current_filter->value,5)) ||
		    	    (!current_filter->is && g_strncasecmp(manufacturer_fields[0],(gchar *)current_filter->value,5))
		    		)
			{
				g_strfreev(manufacturer_fields);
				return TRUE;
			}
		}
		g_strfreev(manufacturer_fields);
	}
	return FALSE;
}

gchar **get_manufacturers (RomEntry * rom)
{
	gchar /**manufacturer1 = NULL,*manufacturer2 = NULL ,*/**manufacturer_fields = NULL;
	manufacturer_fields = g_strsplit(rom->manu,"]",0);
	if (manufacturer_fields[1] != NULL)
	{
		/* we have two winners [Company 1] (company 2)*/
		g_strstrip(g_strdelimit(manufacturer_fields[0],"[",' '));
		g_strstrip(g_strdelimit(manufacturer_fields[1],"()",' '));
	} else
	{
		g_strfreev(manufacturer_fields);
		manufacturer_fields = g_strsplit(rom->manu,"+",0);
		if (manufacturer_fields[1] != NULL)
		{
			/* we have two winners  Company1+Company2*/
			g_strstrip(g_strdelimit(manufacturer_fields[0],"[",' '));
			g_strstrip(g_strdelimit(manufacturer_fields[1],"()",' '));
		} else
		{
			g_strfreev(manufacturer_fields);
			manufacturer_fields = g_strsplit(rom->manu,"/",0);
			if (manufacturer_fields[1] != NULL)
			{
				/* we have two winners  Company1/Company2*/
				g_strstrip(g_strdelimit(manufacturer_fields[0],"[",' '));
				g_strstrip(g_strdelimit(manufacturer_fields[1],"()",' '));
			}
			else
			{
				g_strfreev(manufacturer_fields);
				manufacturer_fields = g_strsplit(rom->manu,"(",0);
				if (manufacturer_fields[1] != NULL)
				{
					/* we have two winners  Company1/Company2*/
					g_strstrip(g_strdelimit(manufacturer_fields[0],"[",' '));
					g_strstrip(g_strdelimit(manufacturer_fields[1],"()",' '));
				}
			}
		}
	}
	return manufacturer_fields;
}


void launch_game(RomEntry *rom)
{
	FILE *xmame_pipe=NULL;
	gchar line[BUFFER_SIZE];
	gchar *opt,*p,*p2;
	gchar *rompath,*samplepath;
	gchar *rompath_option,*samplepath_option,*artworkpath_option,*screenshotdir_option,*hiscoredir_option;
	gchar *mameinfofile_option,*historyfile_option,*hiscorefile_option,*cheatfile_option;			
	gfloat done;
	gint nb_loaded = 0;
	GList *extra_output = NULL, *extra_output2 = NULL;
	gboolean error_during_load;
	GtkWidget *progress_window;
	
	/* prepares options*/
	rompath = g_strjoinv(":",gui_prefs.RomPath);
	if (strlen(rompath)>0)
		rompath_option = g_strdup_printf("-rompath %s",rompath);
	else
		rompath_option = g_strdup(" ");
	g_free(rompath);
	samplepath = g_strjoinv(":",gui_prefs.SamplePath);
	if (strlen(samplepath)>0)
		samplepath_option = g_strdup_printf("-samplepath %s",samplepath);
	else
		samplepath_option = g_strdup(" ");
	g_free(samplepath);
	if (strlen(gui_prefs.ArtworkDirectory)>0)
		artworkpath_option = g_strdup_printf("-artworkpath %s",gui_prefs.ArtworkDirectory);	
	else
		artworkpath_option = g_strdup(" ");
	if (strlen(gui_prefs.SnapshotDirectory)>0)
		screenshotdir_option = g_strdup_printf("-screenshotdir %s",gui_prefs.SnapshotDirectory);	
	else
		screenshotdir_option = g_strdup(" ");
	if (strlen(gui_prefs.HiscoreDirectory)>0)
		hiscoredir_option = g_strdup_printf("-spooldir %s",gui_prefs.HiscoreDirectory);	
	else
		hiscoredir_option = g_strdup(" ");
	if (strlen(gui_prefs.CheatFile)>0)
		cheatfile_option = g_strdup_printf("-cheatfile %s",gui_prefs.CheatFile);	
	else
		cheatfile_option = g_strdup(" ");
	if (strlen(gui_prefs.HiscoreFile)>0)
		hiscorefile_option = g_strdup_printf("-hiscorefile %s",gui_prefs.HiscoreFile);	
	else
		hiscorefile_option = g_strdup(" ");
	if (strlen(gui_prefs.HistoryFile)>0)
		historyfile_option = g_strdup_printf("-historyfile %s",gui_prefs.HistoryFile);	
	else
		historyfile_option = g_strdup(" ");
	if (strlen(gui_prefs.MameInfoFile)>0)
		mameinfofile_option = g_strdup_printf("-mameinfofile %s",gui_prefs.MameInfoFile);	
	else
		mameinfofile_option = g_strdup(" ");
	/* create the command */
        opt=g_strdup_printf("%s %s %s %s %s %s %s %s %s %s %s 2>&1",
			mame_executable,
			rompath_option,
			samplepath_option,
			artworkpath_option,
			screenshotdir_option,
			hiscoredir_option,
			cheatfile_option,
			hiscorefile_option,
			historyfile_option,
			mameinfofile_option,
			rom->romname);
	/*free options*/
	g_free(samplepath_option);
	g_free(rompath_option);
	g_free(artworkpath_option);
	g_free(screenshotdir_option);
	g_free(hiscoredir_option);
	
	progress_window = create_loading_progress_window(TRUE);
	g_snprintf(line,BUFFER_SIZE,_("Loading %s:"),gui_prefs.DefaultGame);
	gtk_window_set_title(GTK_WINDOW(progress_window),line);
	gtk_widget_show_all(progress_window);
	set_icon(progress_window,_("progression"));
	gtk_widget_hide(MainWindow);
	
	gxmame_debug("running command %s",opt);
        xmame_pipe = popen(opt, "r");
	g_message(_("Loading %s:"),rom->gamename);
	/* Loading */
	while (fgets(line, BUFFER_SIZE, xmame_pipe))
	{
		gxmame_debug("xmame: %s",line);
		if (!strncmp(line,"loading",7))
		{
			/* remove the last \n */
			for(p=line;(*p && (*p != '\n'));p++);
			*p='\0';p++;
			nb_loaded++;
			/*search for the : */
			for(p=line;(*p && (*p != ':'));p++);
			p=p+2;
			for(p2=p;(*p2 && (*p2 != '\n'));p2++);
			p2='\0';
			done = (gfloat) ((gfloat) (nb_loaded) /
					(gfloat) (rom->nb_roms));
			//g_message("\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b%6.2f%%",done * 100 );
			update_loading_progress_window (done,p);
			while (gtk_events_pending()) gtk_main_iteration();
		}
		if (!strncmp(line,"done",4))
			break;
	}
	gtk_widget_destroy(progress_window);
	while (gtk_events_pending()) gtk_main_iteration();
	/*check if errors */
	error_during_load = FALSE;
	while (fgets(line, BUFFER_SIZE, xmame_pipe))
	{
		for(p=line;(*p && (*p != '\n'));p++);
		*p='\0';p++;
		gxmame_debug("xmame: %s",line);
		if (!strncmp(line,"ERROR",5))
		{
			error_during_load = TRUE;
			extra_output = g_list_append(extra_output,g_strdup(line));
			break;
		}
		extra_output = g_list_append(extra_output,g_strdup(line));
		
	}
	
	pclose(xmame_pipe);
	if (error_during_load)
	{
		int size;
		char **message = NULL;
		gxmame_debug("error during load");
		size = g_list_length(extra_output)+1;
		message = g_new(gchar *,size);
		for(g_list_first(extra_output),size=0;extra_output != NULL;)
		{
//			gxmame_debug("Error:%s",(char *)extra_output->data);
			message[size++] = extra_output->data;
//			extra_output2 = extra_output;
			extra_output = g_list_next(extra_output);
//			g_free(extra_output2->data);
//			g_list_remove(extra_output2,extra_output2->data);

		}
		message[size++] = NULL;
		GXMame_error_message(g_strjoinv("\n",message));
		g_strfreev(message);
		
		/* free the g_list but it seems that since extra_output point to NULL. the list is not here anymore
		  and since we freed the array, textes are not here anymore but the next and previous pointer?*/
		g_list_foreach(extra_output,(GFunc)g_free,NULL);
		g_list_free (extra_output);
		
		/* update game informations */
		rom->has_roms=0;
		
	}
	else
	{
		gxmame_debug("game over");
		for(g_list_first(extra_output);extra_output != NULL;)
		{
			extra_output2 = extra_output;
			extra_output = g_list_next(extra_output);
			g_free(extra_output2->data);
			g_list_remove(extra_output2,extra_output2->data);
		}
		
		/* update game informations */
		rom->timesplayed++;
		rom->has_roms=1;
	}
	gtk_widget_show(MainWindow);
        g_free(opt);
	/* update the gui for the times played and romstatus if there was any error */
	update_game_in_list(rom->position);
}

void exit_gxmame(void)
{
	g_message(_("Exiting GXMame..."));
	/* prevent to erase games preference if the game list was not loaded
	  (gamelist file corruption or rebuild of a list with a bogus executable)*/
	if (roms)
		save_games_ini();
	save_gxmame_ini();
	save_dirs_ini();
	save_gxmamerc();
	gtk_main_quit();
	gtk_exit (0);
}


gint compare_game_name(RomEntry *Rom1,RomEntry *Rom2)
{
	gint result=0;
	if ((!Rom1->the_trailer && !Rom2->the_trailer) ||
	    (Rom1->the_trailer && Rom2->the_trailer)||
			gui_prefs.ModifyThe)
	{
		if (!(result=strcmp(Rom1->gamename,Rom2->gamename)))
			result=strcmp(Rom1->gamenameext,Rom2->gamenameext);
	}
	else if (Rom1->the_trailer && !Rom2->the_trailer)
		result=strncmp("The ",Rom2->gamename,4);
	else if (!Rom1->the_trailer && Rom2->the_trailer)
		result=strncmp(Rom1->gamename,"The ",4);
	else
		result=0;
	return(result);
}

void destroy_simple_filter (GtkCTree *ctree,
                            GtkCTreeNode *node,
                            gpointer data)
{
	simple_filter *node_data;
	
	/* free a simple filter */
	/* note: value for manufacturer, (year?), categories, and version 
	   will be freed when I free glist 
	   (at the condition that GTK accept to give my memory back for glists)*/
	node_data = gtk_ctree_node_get_row_data (ctree,node);
	if((node_data->type)==CLONE ||		
	   (node_data->type)==CLONE ||		
	   (node_data->type)==DRIVER ||		
	   (node_data->type)==CONTROL)
	{
		g_free(node_data->value);		
	}
		
	g_free(node_data);
	gtk_ctree_remove_node(ctree,node);
}


/*void free_game_glist_element  (gpointer data,
                               gpointer user_data)
{
	RomEntry *tmprom;
	tmprom = (RomEntry *)data;
	g_free(tmprom);
}
*/
