/*
 * GXMame
 *
 * 2002, Stphane Pontier <shadow_walker@users.sourceforge.net>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "support.h"
#include "gui.h"

//#define DEBUG

#define COLUMN_COUNT 25
#define MAX_COLUMN_NAME 30
#define SELECT_TIMEOUT 100

#define EXECUTABLE 1
#define GAMELIST 2

/* Maximum length of some strings */
#define MAX_ROMNAME 10
#define MAX_GAMENAME 100
#define MAX_MANU 50
#define MAX_YEAR 5
#define MAX_STATUS 20
#define MAX_CPU 20
#define MAX_CONTROL 20
#define MAX_CATEGORY 40
#define MAX_NUMPLAYERS 3
#define MAX_NUMBUTTONS 3
#define MAX_MAMEVER 20
#define MAX_FAVORITE 10
#define MAX_DRIVERSTATUS 20
#define MAX_TIMESPLAYED 20
#define MAX_PATHNAME 100
#define NB_CPU 4

typedef enum _Columns_type {
	GAMENAME,
	HAS_ROMS,
	HAS_SAMPLES,
	ROMNAME,
	VECTOR,
	CONTROL,
	TIMESPLAYED,
	MANU,
	YEAR,
	CLONE,
	DRIVER,
	STATUS,
	ROMOF,
	DRIVERSTATUS,
	NUMPLAYERS,
	NUMBUTTONS,
	CPU1,
	CPU2,
	CPU3,
	CPU4,
	SOUND1,
	SOUND2,
	SOUND3,
	SOUND4,
	MAMEVER,
	CATEGORY,
	FAVORITE,
	CHANNELS
} Columns_type;

typedef enum _RomStatus {
	CORRECT,
	INCORRECT,
	MISSING,
	MAY_WORK
} RomStatus;

typedef enum _folder_filters_list {
	NULL_FOLDER_ID,	/* 0	just for the folderID compliance*/
	ALL,		/* 1 */
	AVAILABLE,	/* 2 */
	UNAVAILABLE,	/* 3 */
	NEOGEO,		/* 4 */
	MANUFACTURERS, 	/* 5 just for the folderID compliance*/
	YEARS, 		/* 6 just for the folderID compliance*/
	WORKING,	/* 7 */
	NONWORKING,	/* 8 */
	CUSTOMS, 	/* 9 just for the folderID compliance*/
	  PLAYED,	/* 10 */
	  FAVORITES,	/* 11 */
	ORIGINALS,	/* 12 */
	CLONES,		/* 13 */
	RASTERS,	/* 14 */
	VECTORS,	/* 15 */
	TRACKBALL,	/* 16 */
	STEREO,		/* 17 */
	CPS1,		/* 18 */
	CPS2,		/* 19 */
	CATEGORIES,	/* 20 */
	VERSIONS	/* 21 */
} folder_filters_list;
#define NUMBER_FOLDER 22

typedef enum _ListMode {
	LARGE_ICONS,
	SMALL_ICONS,
	LIST,
	DETAILS
} ListMode;

typedef struct _RomEntry {
	gchar romname[MAX_ROMNAME];
	gchar gamename[MAX_GAMENAME];
	gchar gamenameext[MAX_GAMENAME];
	gchar year[MAX_YEAR];
	gchar manu[MAX_MANU];
	gchar cloneof[MAX_ROMNAME];
	gchar romof[MAX_ROMNAME];
	gchar sampleof[MAX_ROMNAME];
	gchar driver[MAX_ROMNAME];
	gchar cpu1[MAX_CPU];
	gchar cpu2[MAX_CPU];
	gchar cpu3[MAX_CPU];
	gchar cpu4[MAX_CPU];
	gboolean cpu1_sound_flag;
	gboolean cpu2_sound_flag;
	gboolean cpu3_sound_flag;
	gboolean cpu4_sound_flag;
	guint cpu1_clock;
	guint cpu2_clock;
	guint cpu3_clock;
	guint cpu4_clock;
	gchar sound1[MAX_CPU];
	gchar sound2[MAX_CPU];
	gchar sound3[MAX_CPU];
	gchar sound4[MAX_CPU];
	guint sound1_clock;
	guint sound2_clock;
	guint sound3_clock;
	guint sound4_clock;
	gchar control[MAX_CONTROL];
	gchar category[MAX_CATEGORY];
	gchar mame_ver_added[MAX_MAMEVER];
	gint num_players;
	gint num_buttons;
	gint channels;
	gboolean vector;
	gboolean status;
	
	gboolean horizontal;
	guint screen_x;
	guint screen_y;
	gfloat screen_freq;
	guint colors;

	gint nb_roms;
	gint nb_samples;
	gboolean the_trailer;

	gint timesplayed;
	gint has_roms;
	gint has_samples;
	gboolean favourite;
	
	/* position of the game in the GTKCList */
	gint position;
} RomEntry;

struct gui_prefs_struct {
	gchar DefaultGame[MAX_GAMENAME];
	RomEntry *DefaultGame_p;
	/* 1:All, 2:available, 3=unavailable, 4=NeoGeo, 5:Manufacturers
	   6=Years, 7=Working, 8=NonWorking, 9=Custom, 10=Played, 11=Favorites
	   12=originals, 13=clones 14=Raster, 15=Vector, 16=Trackball, 17=Stereo
	   18=CPS1, 19=CPS2
	   Mame32K			| GXMame
	   20=Namco 21=hack...		| 22=3D Games
	   got the year in good order, just need to find the good ofsset given by the number of manufacturer
	   240=year>1980 		| 293=year>1980
	   ...				| ...
	   248=<U>			| 301=<U>
	   249=1986			| 302=1986
	   ...				| ...
	   266=2000			| 319=2000
	  will be able to stick with mame32k only with first folders or 
	  I have to assign a folder ID to all manufacturer and year at 
	  the condition that I got the same manufacturers list.*/
	folder_filters_list FolderID;
	/*Show screenshot panel*/
	gint ShowScreenShot;
	/* 0:Snaps, 1:Flyers, 2:Cabinets, 3:Marquees*/
//	gint ShowFlyer;
	gint ShowToolBar;
	gint ShowStatusBar;
	gint ShowFolderList;
//	gint GameCheck;
//	gint VersionCheck;
//	gint MMXCheck;
	gboolean ModifyThe;
//	gint UseFavoriteNetPlay;
//	gint UseIMEInChat;
//	gint ChatDrawMode;
	gint SortColumn;
	gint SortReverse;
	gint GUIPosX;
	gint GUIPosY;
	gint GUIWidth;
	gint GUIHeight;
/*set by local
	gchar *Language;*/
	ListMode current_mode;
	ListMode previous_mode;

	gint Splitters[2];
//	gchar *ListFont;
	/* GList or array? */
	GList *ColumnWidths;
	GList *ColumnOrder;
	GList *ColumnShown;
	gchar *SaveVersion;
//?	gint FontColor;
	gint ResetGUI;
	gint ResetGameDefaults;
	
	/* directories specification */
	gchar **RomPath;
	gchar **SamplePath;
	gchar *ArtworkDirectory;
	gchar *SnapshotDirectory;
	gchar *InputDirectory;
	gchar *HiscoreDirectory;
			
	/* specifics option to GXMame (path to catver, common path...)
	   not in dir.ini */
	gchar *catverDirectory;
	gchar *CheatFile;
 	gchar *HiscoreFile;
 	gchar *HistoryFile;
 	gchar *MameInfoFile;
};

struct main_gui_struct {
	GtkMenu *popup_menu;
	GtkWidget *popup_play_menu;
	GtkWidget *popup_add_to_favorites;
	GtkWidget *popup_remove_from_favorites;
	
	GtkMenuItem *play_menu;
	GtkMenuItem *add_to_favorites;
	GtkMenuItem *remove_from_favorites;
	GtkCheckMenuItem *folder_list_menu;
	GtkCheckMenuItem *screen_shot_menu;
	GtkCheckMenuItem *large_icons_view_menu;
	GtkCheckMenuItem *small_icons_view_menu;
	GtkCheckMenuItem *list_view_menu;
	GtkCheckMenuItem *details_view_menu;
	GtkCheckMenuItem *toolbar_view_menu;
	GtkCheckMenuItem *status_bar_view_menu;

	GtkWidget *hseparator2;
	GtkToolbar *toolbar;
	GtkWidget *hseparator1;
	GtkToggleButton *filterShowButton;
	GtkToggleButton *snapShowButton;
	GtkToggleButton *large_icons_view_button;
	GtkToggleButton *small_icons_view_button;
	GtkToggleButton *list_view_button;
	GtkToggleButton *details_view_button;
	
	GtkWidget *combo_progress_bar;
	GtkStatusbar *status_progress_bar;
	GtkProgressBar *progress_progress_bar;
	GtkWidget *tri_status_bar;
	GtkStatusbar *statusbar1;
	GtkStatusbar *statusbar2;
	GtkStatusbar *statusbar3;
	
	GtkPaned *hpanedLeft;
	GtkPaned *hpanedRight;
	
	GtkWidget *scrolled_window_filters;
	GtkWidget *scrolled_window_games;
	GtkCTree *ctree_filters;
	GtkFrame *screenshot_hist_frame;
	GtkBox *screenshot_hist_vbox;
	GtkWidget *testgtkpix;
	GtkWidget *history_box;
	GtkWidget *displayed_list;
};


typedef struct _simple_filter {
	folder_filters_list FolderID;
	Columns_type type;
	gboolean is;
	gpointer *value;
} simple_filter;

typedef struct _xmame_cversion {
	gchar *name;
	gchar *target;
	gchar *version;
} xmame_cversion;


/* global variables */
xmame_cversion executable_version;
xmame_cversion list_version;
gchar *mame_executable;
gchar *mame_exe_version,*mame_exe_name;
//gchar *xmame_target;
//gchar *list_exe_mame_version,*list_exe_mame_name;
guint supported_games, visible_games,supported_samples_games;
GList *roms;
GList *years;
GList *manufacturers;
GList *categories;
GList *versions;
GList *Columns;
simple_filter *current_filter;


GList *not_checked_list;
struct main_gui_struct main_gui;
struct gui_prefs_struct gui_prefs;

void gxmame_debug(const gchar * format, ...);
gboolean game_filtered (RomEntry * rom);
gchar **get_manufacturers (RomEntry * rom);
void launch_game(RomEntry *rom);
void exit_gxmame(void);

gint compare_game_name(RomEntry *Rom1,RomEntry *Rom2);

void destroy_simple_filter (GtkCTree *ctree,
                            GtkCTreeNode *node,
                            gpointer data);
/*void free_game_glist_element  (gpointer data,
                               gpointer user_data);*/
