/*
 * GXMame
 *
 * 2002, Stphane Pontier <shadow_walker@users.sourceforge.net>
 * 
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */

#include "gxmame.h"
#include "io.h"
#include "progression_window.h"
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/stat.h>

int compare_year(gint *Y1,gint *Y2)
{
	int temp;
	temp = (Y1>Y2)?1:0;
	return temp;
}

gboolean load_games_list()
{
	FILE *gamelist;
	gint romindex, i, count;
	GList *listpointer;
	
	gint year;

	gchar *filename;
	gchar line[2000];
	gchar **tmp_array = NULL;
	gchar *tmp, *p;
	RomEntry *rom;
	gboolean exe_version_checked = FALSE, manufacturer_in_list;
	gchar *manufacturer1,*manufacturer2,**manufacturer_fields = NULL;

	supported_samples_games = 0;
	romindex = 0;
	g_message(_("Loading gamelist"));

	filename = g_malloc(200);

	/* clean the existing list if any */
	g_list_foreach(roms, (GFunc) g_free, NULL);
	g_list_free(roms);
	roms = NULL;

	g_snprintf(filename, 200, "%s/.gxmame/gamelist",
		   getenv("HOME"));
	gamelist = fopen(filename, "r");
	if (gamelist == NULL) {
		g_free(filename);
		return (FALSE);
	}

	while (fgets(line, 500, gamelist)) {
		count = 0;
		i = 0;

		p = line;
		tmp = p;

		/* Skip comments */
		if (*tmp != '#') {
			while (*tmp && (*tmp++ != '\n')) {
				i++;
				count++;
			}
			tmp_array = g_strsplit(p, "", 38);
			rom = g_malloc(sizeof(RomEntry));
			if (rom == NULL)
				g_warning
				    ("Out of memory while loading gamelist");

			g_snprintf(rom->romname, MAX_ROMNAME,
				   "%s", tmp_array[0]);
			g_snprintf(rom->gamename, MAX_GAMENAME,
				   "%s", tmp_array[1]);
			g_snprintf(rom->gamenameext, MAX_GAMENAME,
				   "%s", tmp_array[2]);
			if (!strcmp(tmp_array[3], "true"))
				rom->the_trailer = TRUE;
			else 
				rom->the_trailer = FALSE;

			g_snprintf(rom->year, MAX_YEAR, "%s",
				   tmp_array[4]);
			g_snprintf(rom->manu, MAX_MANU, "%s",
				   tmp_array[5]);
			g_snprintf(rom->cloneof, MAX_ROMNAME,
				   "%s", tmp_array[6]);
			g_snprintf(rom->romof, MAX_ROMNAME,
				   "%s", tmp_array[7]);
			g_snprintf(rom->driver, MAX_ROMNAME,
				   "%s", tmp_array[8]);

			if (!strcmp(tmp_array[9], "true"))
				rom->status = TRUE;
			else 
				rom->status = FALSE;
			rom->colors = atoi(tmp_array[10]);
			
			/* CPU */
			rom->cpu1_sound_flag = rom->cpu2_sound_flag = rom->cpu3_sound_flag = rom->cpu4_sound_flag = FALSE;
			if (!strncmp(tmp_array[11], "(sound)",7)) {
				p = tmp_array[11]; p=p+7;
				g_snprintf(rom->cpu1, MAX_CPU,
					   "%s", p);
				rom->cpu1_sound_flag = TRUE;
			}
			else
				g_snprintf(rom->cpu1, MAX_CPU,
					   "%s", tmp_array[11]);
			rom->cpu1_clock = atoi(tmp_array[12]);
			
			if (!strncmp(tmp_array[13], "(sound)",7)) {
				p = tmp_array[13]; p=p+7;
				g_snprintf(rom->cpu2, MAX_CPU,
					   "%s", p);
				rom->cpu2_sound_flag = TRUE;
			}
			else
				g_snprintf(rom->cpu2, MAX_CPU,
					   "%s", tmp_array[13]);
			rom->cpu2_clock = atoi(tmp_array[14]);
			
			if (!strncmp(tmp_array[15], "(sound)",7)) {
				p = tmp_array[15]; p=p+7;
				g_snprintf(rom->cpu3, MAX_CPU,
					   "%s", p);
				rom->cpu3_sound_flag = TRUE;
			}
			else
				g_snprintf(rom->cpu3, MAX_CPU,
					   "%s", tmp_array[15]);
			rom->cpu3_clock = atoi(tmp_array[16]);
			
			if (!strncmp(tmp_array[17], "(sound)",7)) {
				p = tmp_array[17]; p=p+7;
				g_snprintf(rom->cpu4, MAX_CPU,
					   "%s", p);
				rom->cpu4_sound_flag = TRUE;
			}
			else
				g_snprintf(rom->cpu4, MAX_CPU,
					   "%s", tmp_array[17]);
			rom->cpu4_clock = atoi(tmp_array[18]);
			
			
			
			
			if (strcmp(tmp_array[19], "")) {
				g_snprintf(rom->sound1, MAX_CPU,
					   "%s", tmp_array[19]);
			}
			rom->sound1_clock = atoi(tmp_array[20]);
			
			if (strcmp(tmp_array[21], "")) {
				g_snprintf(rom->sound2, MAX_CPU,
					   "%s", tmp_array[21]);
			}
			rom->sound2_clock = atoi(tmp_array[22]);
			
			if (strcmp(tmp_array[23], "")) {
				g_snprintf(rom->sound3, MAX_CPU,
					   "%s", tmp_array[23]);
			}
			rom->sound3_clock = atoi(tmp_array[24]);
			
			if (strcmp(tmp_array[25], "")) {
				g_snprintf(rom->sound4, MAX_CPU,
					   "%s", tmp_array[25]);
			}
			rom->sound4_clock = atoi(tmp_array[26]);
			
			rom->num_players = atoi(tmp_array[27]);
			rom->num_buttons = atoi(tmp_array[28]);
			if (strcmp(tmp_array[29], "")) {
				g_snprintf(rom->control,
					   MAX_CONTROL, "%s",
					   tmp_array[29]);
			}
			if (!strcmp(tmp_array[30], "true"))
				rom->vector = TRUE;
			else
				rom->vector = FALSE;

			rom->screen_x = atoi(tmp_array[31]);
			rom->screen_y = atoi(tmp_array[32]);
			rom->screen_freq = atoi(tmp_array[33]);
			if (!strcmp(tmp_array[34], "horizontal"))
				rom->horizontal = TRUE;
			else
				rom->horizontal = FALSE;
			
			rom->channels = atoi(tmp_array[35]);
			rom->nb_roms = atoi(tmp_array[36]);
			rom->nb_samples = atoi(tmp_array[37]);
			if  (rom->nb_samples>0)
				supported_samples_games++;
			g_strfreev(tmp_array);
			
			/*initialize timesplayed and favorite fields*/
			rom->timesplayed = 0;
			rom->favourite = 0;

			/*generate Glist for years*/
			year = atoi(rom->year);
			if  (!g_list_find(years, GINT_TO_POINTER(year)))
			{
				years = g_list_prepend(years, GINT_TO_POINTER(year));
			}

			/*generate glist for manufacturers*/
			manufacturer_fields = get_manufacturers (rom);
			manufacturer1 = g_strdup(manufacturer_fields[0]);
			manufacturer_in_list = FALSE;
			listpointer = g_list_first(manufacturers);
			while ((listpointer != NULL))
			{
				if (!strncmp(listpointer->data,manufacturer1,5))
				{
					manufacturer_in_list = TRUE;
					break;
				}
				listpointer = g_list_next(listpointer);
			}
			if (!manufacturer_in_list)
			{
			 	manufacturers = g_list_prepend(manufacturers, manufacturer1);
//			 	manufacturers = g_list_insert_sorted(manufacturers, manufacturer1, (GCompareFunc )g_strcasecmp);
			}
			if (manufacturer_fields[1] != NULL)
			{
				manufacturer2 = g_strdup(manufacturer_fields[1]);
				manufacturer_in_list = FALSE;
				listpointer = g_list_first(manufacturers);
				while ((listpointer != NULL))
				{
					if (!strncmp(listpointer->data,manufacturer2,5))
					{
						manufacturer_in_list = TRUE;
						break;
					}
					listpointer = g_list_next(listpointer);
				}
				if (!manufacturer_in_list)
				{
				 	manufacturers = g_list_prepend(manufacturers, manufacturer2);
				}
			}
			g_strfreev(manufacturer_fields);
			
			/* fill the some fields with default value if we have problems
			   with game.ini after */
			rom->timesplayed = 0;
			rom->has_roms = 2;
			rom->has_samples = 2;
			rom->favourite = FALSE;

			/* fill the some fields with default value if we have problems
			   with catver after */
			g_snprintf(rom->category,MAX_CATEGORY,"Unknown");
			g_snprintf(rom->mame_ver_added,MAX_MAMEVER,"Unknown");

			roms = g_list_insert_sorted(roms, (gpointer) rom, (GCompareFunc )compare_game_name);
		} else if (!exe_version_checked) {
			/* Check for the GXMame version in the top comment line */
			while (*tmp && (*tmp++ != '\n')) {
				i++;
				count++;
			}
			p[i]='\0';
			tmp_array = g_strsplit(p, " ", 3);

			gxmame_debug("Checking version of gamelist file: %s - %s",tmp_array[1], tmp_array[2]);

			if (strcmp(tmp_array[1], "GXMame")) {
				g_message(_("Could not recognize format of gamelist.\nYou need to build a new gamelist."));
				list_version.version = g_strdup("unknown");
				g_strfreev(tmp_array);
				return (FALSE);
			} else if (g_strtod(tmp_array[2], NULL) < 0.7) {
				g_message(_("Gamelist from too old GXMame\nThe format of the gamelist has changed in this version.\nYou need to build a new gamelist."));
				list_version.version = g_strdup("too old");
				g_strfreev(tmp_array);
				return (FALSE);
			}
			g_strfreev(tmp_array);
			exe_version_checked = TRUE;
		} else {
			/* Check for the GXMame version in the top comment line */
			while (*tmp && (*tmp++ != '\n')) {
				i++;
				count++;
			}
			p[i]='\0';
			gxmame_debug("reading comment: %s",p);

			if (!strncmp(p, "# Name",6))
			{
				p = p+7;
				list_version.name = g_strdup(p);
			} else if (!strncmp(p, "# Target",8))
			{
				p = p+9;
				list_version.target = g_strdup(p);
			} else if (!strncmp(p, "# Version",9))
			{ 
				p = p+10;
				list_version.version = g_strdup(p);
			}
			
/*			if (strncmp(p, "# Version",9)) {
				g_message(_("Could not recognize format of gamelist.\nYou need to build a new gamelist."));
				list_exe_mame_version = g_strdup("unknown");
				return (FALSE);
			} else  {
				p = p+10;
				list_exe_mame_version = g_strdup(p);
			}*/
		}
	}

	fclose(gamelist);
	g_free(filename);

	supported_games = g_list_length(roms);
	g_message(_("Loaded %d roms by %d manufacturers covering %d years."),g_list_length(roms),g_list_length(manufacturers),g_list_length(years));
	g_message(_("with %d games supporting samples."),supported_samples_games);

	return (TRUE);
}


gboolean load_games_ini(void)
{
	FILE *gameini_list;
	gchar *filename;
	gchar line[2000];
	gchar **option_value = NULL;
	GList *listpointer;
	RomEntry *tmprom = NULL;
	gboolean game_found=FALSE;
	
	filename = g_strdup_printf("%s/.gxmame/games.ini",getenv("HOME"));	
	gameini_list = fopen(filename, "r");
	if (gameini_list == NULL)
	{
		gxmame_debug("games.ini not found");
		g_free(filename);
		not_checked_list = g_list_copy (roms);
		return (FALSE);
	}
	while (fgets(line, 500, gameini_list))
	{
		if(*line =='[')
		{	
			g_strstrip(g_strdelimit(line,"[]\r\n",' '));
			/* ok only if mame32k games,ini doesn't change */
			/* find the the rom in the glist */
			for(listpointer = g_list_first(roms), game_found = FALSE;
					(listpointer != NULL);
					listpointer = g_list_next(listpointer))
			{
				tmprom = (RomEntry *) listpointer->data;
				if (!strcmp(tmprom->romname,line))
				{
					game_found = TRUE;
					break;
				}
			}
			if (game_found)
			{
				while (fgets(line, 500, gameini_list))
				{/* need to test the carriage return if the file come from Mame32K under windows*/
					if(*line =='\r' ||*line =='\n')
						break;
					option_value = g_strsplit(g_strstrip(g_strdelimit(line,"[]\r\n",' ')),"=",0);
					if (option_value[1] != NULL)
					{	if (!strcmp(option_value[0],"PlayCount"))
						{	tmprom->timesplayed = atoi(option_value[1]);
						} else if (!strcmp(option_value[0],"HasRoms"))
						{	tmprom->has_roms = atoi(option_value[1]);
						} else if (!strcmp(option_value[0],"HasSamples"))
						{	tmprom->has_samples = atoi(option_value[1]);
						} else if (!strcmp(option_value[0],"Favorite"))
						{	tmprom->favourite = atoi(option_value[1])?TRUE:FALSE;
						}else if (!strcmp(option_value[0],"AutofireDelay"))
						{	/*store this somewhere to be able to put it again
							  when we will save the game.ini*/
						}
					g_strfreev(option_value);
					}
				}
				if(tmprom->has_roms == 2)
				{
//					printf("a tester%s (%s)\n",tmprom->gamename,tmprom->romname);
					not_checked_list = g_list_append(not_checked_list, tmprom);
				}
			}
		}
		
	}
	fclose(gameini_list);
	g_free(filename);
	return TRUE;
}


gboolean load_catver_ini(void)
{
	FILE *catver_file;
	gchar *filename;
	gchar line[2000];
	gchar **game_value = NULL;
	GList *listpointer;
	RomEntry *tmprom = NULL;
	gboolean game_found=FALSE;
	gchar *category;
	gchar *version;
	gboolean category_in_list, version_in_list;
	
	filename = g_strdup_printf("%scatver.ini",gui_prefs.catverDirectory);	
	catver_file = fopen(filename, "r");
	if (catver_file == NULL)
	{
		gxmame_debug("catver.ini not found in %s",filename);
		g_free(filename);
		return (FALSE);
	}
	fgets(line, 500, catver_file);
	if (*line!=';')
	{
		g_free(filename);
		return FALSE;
	}
	g_strstrip(g_strdelimit(line,"\r\n",' '));
	gxmame_debug("Loading catver: %s",line);
	fgets(line, 500, catver_file);
	fgets(line, 500, catver_file);
	if(strncmp(line,"[Category]", 10))
	{
		g_free(filename);
		return FALSE;
	}
		
	while (fgets(line, 500, catver_file))
	{
		/*end of categories */
		if (*line=='\r')
			break;
		g_strstrip(g_strdelimit(line,"\r\n",' '));
		game_value = g_strsplit(line,"=",2);
		for(listpointer = g_list_first(roms), game_found = FALSE;
				(listpointer != NULL);
				listpointer = g_list_next(listpointer))
		{
			tmprom = (RomEntry *) listpointer->data;
			if (!strcmp(tmprom->romname,game_value[0]))
			{
				game_found = TRUE;
				break;
			}
		}
		if (game_found)
		{
			g_snprintf(tmprom->category,MAX_CATEGORY, game_value[1]);
		}
		/* create the glist for categories */
		category_in_list = FALSE;
		listpointer = g_list_first(categories);
		while ((listpointer != NULL))
		{
			if (!strcmp(listpointer->data,game_value[1]))
			{
				category_in_list = TRUE;
				break;
			}
			listpointer = g_list_next(listpointer);
		}
		if (!category_in_list)
		{
			category = g_strdup(game_value[1]);
		 	categories = g_list_prepend(categories, category);
		}
		g_strfreev(game_value);
	}
	
	fgets(line, 500, catver_file);
	/* and now come the versiona*/
	if(strncmp(line,"[VerAdded]", 10))
	{
		printf("read:%s\n",line);
		g_free(filename);
		return FALSE;
	}
		
	while (fgets(line, 500, catver_file))
	{
		/*end of version */
		if (*line=='\r')
			break;
		g_strstrip(g_strdelimit(line,"\r\n",' '));
		game_value = g_strsplit(line,"=",2);
		for(listpointer = g_list_first(roms), game_found = FALSE;
				(listpointer != NULL);
				listpointer = g_list_next(listpointer))
		{
			tmprom = (RomEntry *) listpointer->data;
			if (!strcmp(tmprom->romname,game_value[0]))
			{
				game_found = TRUE;
				break;
			}
		}
		if (game_found)
		{
			g_snprintf(tmprom->mame_ver_added,MAX_MAMEVER, game_value[1]);
		}
		/* create the glist for versions */
		version_in_list = FALSE;
		listpointer = g_list_first(versions);
		while ((listpointer != NULL))
		{
			if (!strcmp(listpointer->data,game_value[1]))
			{
				version_in_list = TRUE;
				break;
			}
			listpointer = g_list_next(listpointer);
		}
		if (!version_in_list)
		{
			version = g_strdup(game_value[1]);
		 	versions = g_list_prepend(versions, version);
		}
		g_strfreev(game_value);
	}
	fclose(catver_file);
	g_free(filename);
	return TRUE;
}


gboolean load_gxmame_ini(void)
{
	FILE *gameini_list;
	gchar *filename;
	gchar line[2000];
	gchar **option_value = NULL;
	gchar **option_value2 = NULL;
	
	g_snprintf(gui_prefs.DefaultGame,MAX_GAMENAME,"%s","Pac-Man (Midway)");
	gui_prefs.FolderID = 1;
	gui_prefs.ShowScreenShot = 1;
	gui_prefs.ShowToolBar = 1;
	gui_prefs.ShowStatusBar = 1;
	gui_prefs.ShowFolderList = 1;
	gui_prefs.SortColumn = 0;
	gui_prefs.SortReverse = 0;
	gui_prefs.GUIPosX = 0;
	gui_prefs.GUIPosY = 0;
	gui_prefs.GUIWidth = 640;
	gui_prefs.GUIHeight = 400;
	gui_prefs.previous_mode = 3;
	gui_prefs.current_mode = 3;
	gui_prefs.Splitters[0] = 150;
	gui_prefs.Splitters[1] = 300;
	gui_prefs.ModifyThe = TRUE;
      
	
	filename = g_strdup_printf("%s/.gxmame/gxmame.ini",getenv("HOME"));	
	gameini_list = fopen(filename, "r");
	if (gameini_list == NULL)
	{
		gxmame_debug("gxmame.ini not found");
		g_free(filename);
		return (FALSE);
	}
	/* Be careful of the \r\n terminal due to windows */
	while (fgets(line, 500, gameini_list))
	{
		if (!strncmp(line,"[default]",9))
		{	
			while (fgets(line, 500, gameini_list))
			{/* need to test the carriage return if the file come from Mame32K under windows*/
				if(*line =='\r' ||*line =='\n')
					break;
				option_value = g_strsplit(g_strstrip(g_strdelimit(line,"[]\r\n",' ')),"=",0);
				if (option_value[1] != NULL)
				{	if (!strcmp(option_value[0],"DefaultGame"))
					{	g_snprintf(gui_prefs.DefaultGame,MAX_GAMENAME,"%s",option_value[1]);
					} else if (!strcmp(option_value[0],"FolderID"))
					{	gui_prefs.FolderID = atoi(option_value[1]);
					} else if (!strcmp(option_value[0],"ShowScreenShot"))
					{	gui_prefs.ShowScreenShot = atoi(option_value[1]);
					} else if (!strcmp(option_value[0],"ShowToolBar"))
					{	gui_prefs.ShowToolBar = atoi(option_value[1]);
					} else if (!strcmp(option_value[0],"ShowStatusBar"))
					{	gui_prefs.ShowStatusBar = atoi(option_value[1]);
					} else if (!strcmp(option_value[0],"ModifyThe"))
					{	gui_prefs.ModifyThe = atoi(option_value[1]);
					} else if (!strcmp(option_value[0],"ShowFolderList"))
					{	gui_prefs.ShowFolderList = atoi(option_value[1]);
					} else if (!strcmp(option_value[0],"SortColumn"))
					{	gui_prefs.SortColumn = atoi(option_value[1]);
					} else if (!strcmp(option_value[0],"SortReverse"))
					{	gui_prefs.SortReverse = atoi(option_value[1]);
					} else if (!strcmp(option_value[0],"GUIPosX"))
					{	gui_prefs.GUIPosX = atoi(option_value[1]);
					} else if (!strcmp(option_value[0],"GUIPosY"))
					{	gui_prefs.GUIPosY = atoi(option_value[1]);
					} else if (!strcmp(option_value[0],"GUIWidth"))
					{	gui_prefs.GUIWidth = atoi(option_value[1]);
					}else if (!strcmp(option_value[0],"GUIHeight"))
					{	gui_prefs.GUIHeight = atoi(option_value[1]);
					}else if (!strcmp(option_value[0],"ListMode"))
					{	gui_prefs.current_mode = gui_prefs.previous_mode = atoi(option_value[1]);
					}else if (!strcmp(option_value[0],"Splitters"))
					{	
						option_value2 = g_strsplit(option_value[1],",",2);
						gui_prefs.Splitters[0] = atoi(option_value2[0]);
						gui_prefs.Splitters[1] = atoi(option_value2[1]);
						g_strfreev(option_value2);
					}
				g_strfreev(option_value);
				}
			}
		}
	}
	fclose(gameini_list);
	g_free(filename);
	return TRUE;
}

/* this the master config file, it will contain the path to every other config file
   when we will share these config files with another mame32 application */
gboolean load_gxmamerc(void)
{
	FILE *gamerc_file;
	gchar *filename;
	gchar line[2000];
	gchar **option_value = NULL;
	
	/* default values */
	mame_executable = g_strdup("xmame");
	gui_prefs.catverDirectory = g_strdup_printf("%s/.gxmame/",getenv("HOME"));
	gui_prefs.CheatFile = g_strdup("/usr/lib/games/xmame/cheat.dat");
	gui_prefs.HiscoreFile = g_strdup("/usr/lib/games/xmame/hiscore.dat");
	gui_prefs.HistoryFile = g_strdup("/usr/lib/games/xmame/history.dat");
	gui_prefs.MameInfoFile = g_strdup("/usr/lib/games/xmame/mameinfo.dat");   
	
	filename = g_strdup_printf("%s/.gxmame/gxmamerc",getenv("HOME"));	
	gamerc_file = fopen(filename, "r");
	if (gamerc_file == NULL)
	{
		gxmame_debug("gxmamerc not found");
		g_free(filename);
		return (FALSE);
	}

	while (fgets(line, 500, gamerc_file))
	{
		/* skip comments */
		if (*line!='#')
		{	
			if(*line =='\n')
				break;
			option_value = g_strsplit(g_strstrip(g_strdelimit(line,"[]\n",' ')),"=",0);
			if (option_value[1] != NULL)
			{	if (!strcmp(option_value[0],"mame_executable"))
				{
					g_free(mame_executable);
					mame_executable = g_strdup(option_value[1]);
				} else if (!strcmp(option_value[0],"catverDirectory"))
				{
					g_free(gui_prefs.catverDirectory);
					gui_prefs.catverDirectory = g_strdup(option_value[1]);
				} else if (!strcmp(option_value[0],"CheatFile"))
				{	
					g_free(gui_prefs.CheatFile);
					gui_prefs.CheatFile = g_strdup(option_value[1]);
				} else if (!strcmp(option_value[0],"HiscoreFile"))
				{	
					g_free(gui_prefs.HiscoreFile);
					gui_prefs.HiscoreFile = g_strdup(option_value[1]);
				} else if (!strcmp(option_value[0],"HistoryFile"))
				{	
					g_free(gui_prefs.HistoryFile);
					gui_prefs.HistoryFile = g_strdup(option_value[1]);
				} else if (!strcmp(option_value[0],"MameInfoFile"))
				{	
					g_free(gui_prefs.MameInfoFile);
					gui_prefs.MameInfoFile = g_strdup(option_value[1]);
				}
				g_strfreev(option_value);
			}
		}
	}
	fclose(gamerc_file);
	g_free(filename);
	return TRUE;
}


gboolean load_dirs_ini(void)
{
	FILE *dirs_ini_file;
	gchar *filename;
	gchar line[2000];
	gchar **option_value = NULL;
	
	gui_prefs.RomPath = g_strsplit("/usr/lib/games/xmame/roms/",";",2);
	gui_prefs.SamplePath = g_strsplit("/usr/lib/games/xmame/samples/",";",2);
	gui_prefs.SnapshotDirectory = g_strdup("/usr/lib/games/xmame/snap/");
	gui_prefs.ArtworkDirectory = g_strdup("/usr/lib/games/xmame/artwork/");
	gui_prefs.HiscoreDirectory = g_strdup("/usr/lib/games/xmame/hi/");
	gui_prefs.InputDirectory = g_strdup("/usr/lib/games/xmame/inp/");
	filename = g_strdup_printf("%s/.gxmame/dirs.ini",getenv("HOME"));	
	dirs_ini_file = fopen(filename, "r");
	if (dirs_ini_file == NULL)
	{
		gxmame_debug("dirs.ini not found");
		g_free(filename);
		return (FALSE);
	}
	/* Be careful of the \r\n terminal due to windows */
	while (fgets(line, 500, dirs_ini_file))
	{
		if (!strncmp(line,"[directories]",13))
		{	
			while (fgets(line, 500, dirs_ini_file))
			{/* need to test the carriage return if the file come from Mame32K under windows*/
				if(*line =='\r' ||*line =='\n')
					break;
				option_value = g_strsplit(g_strstrip(g_strdelimit(line,"[]\r\n",' ')),"=",0);
				if (option_value[1] != NULL)
				{	if (!strcmp(option_value[0],"RomPath"))
					{	g_strfreev(gui_prefs.RomPath);
						gui_prefs.RomPath = g_strsplit(option_value[1],";",0);
					} else if (!strcmp(option_value[0],"SamplePath"))
					{	g_strfreev(gui_prefs.SamplePath);
						gui_prefs.SamplePath = g_strsplit(option_value[1],";",0);
/*					} else if (!strcmp(option_value[0],"ConfigDirectory"))
					{	g_snprintf(gui_prefs.ConfigDirectory,MAX_PATHNAME,"%s",option_value[1]);
					} else if (!strcmp(option_value[0],"FolderDirectory"))
					{	g_snprintf(gui_prefs.FolderDirectory,MAX_PATHNAME,"%s",option_value[1]);
					} else if (!strcmp(option_value[0],"NVRamDirectory"))
					{	g_snprintf(gui_prefs.NVRamDirectory,MAX_PATHNAME,"%s",option_value[1]);
					} else if (!strcmp(option_value[0],"MemCardDirectory"))
					{	g_snprintf(gui_prefs.MemCardDirectory,MAX_PATHNAME,"%s",option_value[1]);
*/					} else if (!strcmp(option_value[0],"InputDirectory"))
					{	
						g_free(gui_prefs.InputDirectory);
						gui_prefs.InputDirectory = g_strdup(option_value[1]);
					} else if (!strcmp(option_value[0],"HiscoreDirectory"))
					{	
						g_free(gui_prefs.HiscoreDirectory);
						gui_prefs.HiscoreDirectory = g_strdup(option_value[1]);
/*					} else if (!strcmp(option_value[0],"StateDirectory"))
					{	g_snprintf(gui_prefs.StateDirectory,MAX_PATHNAME,"%s",option_value[1]);
*/					} else if (!strcmp(option_value[0],"ArtworkDirectory"))
					{
						g_free(gui_prefs.ArtworkDirectory);
						gui_prefs.ArtworkDirectory = g_strdup(option_value[1]);
					} else if (!strcmp(option_value[0],"SnapshotDirectory"))
					{	
						g_free(gui_prefs.SnapshotDirectory);
						gui_prefs.SnapshotDirectory = g_strdup(option_value[1]);
/*					} else if (!strcmp(option_value[0],"IconDirectory"))
					{	g_snprintf(gui_prefs.IconDirectory,MAX_PATHNAME,"%s",option_value[1]);
					} else if (!strcmp(option_value[0],"WavDirectory"))
					{	g_snprintf(gui_prefs.WavDirectory,MAX_PATHNAME,"%s",option_value[1]);
					} else if (!strcmp(option_value[0],"FlyerDirectroy"))
					{	g_snprintf(gui_prefs.FlyerDirectroy,MAX_PATHNAME,"%s",option_value[1]);
					} else if (!strcmp(option_value[0],"CabinetDirectory"))
					{	g_snprintf(gui_prefs.CabinetDirectory,MAX_PATHNAME,"%s",option_value[1]);
					} else if (!strcmp(option_value[0],"MarqueeDirectory"))
					{	g_snprintf(gui_prefs.MarqueeDirectory,MAX_PATHNAME,"%s",option_value[1]);
*/					}
				g_strfreev(option_value);
				}
			}
		}
	}
	fclose(dirs_ini_file);
	g_free(filename);
	return TRUE;
}

gboolean save_games_ini(void)
{
	FILE *game_ini_file=NULL;
	gchar *filename;
	gchar *romname = NULL;
	gchar *timesplayed;
	gchar *has_roms;
	gchar *has_samples;
	gchar *favourite;
	RomEntry *rom;
	GList *list_pointer;
	
	
	filename = g_strdup_printf("%s/.gxmame/games.ini",getenv("HOME"));	
	
	if ((game_ini_file = fopen(filename, "w")) == NULL)
	{
		gxmame_debug("unable to write games.ini");
		g_free(filename);
		return (FALSE);
	}
	
	romname = g_malloc0(MAX_ROMNAME);
	timesplayed = g_malloc0(5);
	has_roms = g_malloc0(5);
	has_samples = g_malloc0(5);
	favourite = g_malloc0(5);
	
	for (list_pointer = g_list_first(roms);
			(list_pointer != NULL);
			list_pointer=g_list_next(list_pointer))
	{
		rom = (RomEntry *)list_pointer->data;
		g_snprintf(timesplayed,5,"%i",rom->timesplayed);
		g_snprintf(has_roms,5,"%i",rom->has_roms);
		g_snprintf(has_samples,5,"%i",rom->has_samples);
		g_snprintf(favourite,5,"%s",rom->favourite?"1":"0");
		fputc ('[',game_ini_file);
		fputs (rom->romname,game_ini_file);
		fputs ("]",game_ini_file);
		fputs ("\r\nPlayCount=",game_ini_file);
		fputs (timesplayed,game_ini_file);
		fputs ("\r\nHasRoms=",game_ini_file);
		fputs (has_roms,game_ini_file);
		fputs ("\r\nHasSamples=",game_ini_file);
		fputs (has_samples,game_ini_file);
		fputs ("\r\nFavorite=",game_ini_file);
		fputs (favourite,game_ini_file);
		fputs ("\r\n\r\n",game_ini_file);
		
	}
	fclose(game_ini_file);
	g_free(filename);
	g_free(romname);
	g_free(timesplayed);
	g_free(has_roms);
	g_free(has_samples);
	g_free(favourite);
	return TRUE;
}


gboolean save_gxmame_ini(void)
{
	FILE *gxmame_ini_file = NULL;
	gchar *filename;
	gchar *option_string = NULL;
	
	
	filename = g_strdup_printf("%s/.gxmame/gxmame.ini",getenv("HOME"));	
	
	if ((gxmame_ini_file = fopen(filename, "w")) == NULL)
	{
		gxmame_debug("unable to write gxmame.ini");
		g_free(filename);
		return (FALSE);
	}
	
	option_string = g_malloc0(MAX_PATHNAME);
 
	fputs ("[default]",gxmame_ini_file);
	fputs ("\r\nDefaultGame=",gxmame_ini_file);
	fputs (gui_prefs.DefaultGame,gxmame_ini_file);
	
	g_snprintf(option_string,MAX_PATHNAME,"%i",gui_prefs.FolderID);
	fputs ("\r\nFolderID=",gxmame_ini_file);
	fputs (option_string,gxmame_ini_file);
	
	g_snprintf(option_string,MAX_PATHNAME,"%i",gui_prefs.ShowScreenShot);
	fputs ("\r\nShowScreenShot=",gxmame_ini_file);
	fputs (option_string,gxmame_ini_file);
	
	g_snprintf(option_string,MAX_PATHNAME,"%i",gui_prefs.ShowToolBar);
	fputs ("\r\nShowToolBar=",gxmame_ini_file);
	fputs (option_string,gxmame_ini_file);
	
	g_snprintf(option_string,MAX_PATHNAME,"%i",gui_prefs.ModifyThe);
	fputs ("\r\nModifyThe=",gxmame_ini_file);
	fputs (option_string,gxmame_ini_file);

		g_snprintf(option_string,MAX_PATHNAME,"%i",gui_prefs.ShowStatusBar);
	fputs ("\r\nShowStatusBar=",gxmame_ini_file);
	fputs (option_string,gxmame_ini_file);
	
	g_snprintf(option_string,MAX_PATHNAME,"%i",gui_prefs.ShowFolderList);
	fputs ("\r\nShowFolderList=",gxmame_ini_file);
	fputs (option_string,gxmame_ini_file);
	
	g_snprintf(option_string,MAX_PATHNAME,"%i",gui_prefs.SortColumn);
	fputs ("\r\nSortColumn=",gxmame_ini_file);
	fputs (option_string,gxmame_ini_file);
	
	g_snprintf(option_string,MAX_PATHNAME,"%i",gui_prefs.SortReverse);
	fputs ("\r\nSortReverse=",gxmame_ini_file);
	fputs (option_string,gxmame_ini_file);
	gdk_window_get_position(MainWindow->window, &gui_prefs.GUIPosX,&gui_prefs.GUIPosY);
	g_snprintf(option_string,MAX_PATHNAME,"%i",gui_prefs.GUIPosX);
	fputs ("\r\nGUIPosX=",gxmame_ini_file);
	fputs (option_string,gxmame_ini_file);
	
	g_snprintf(option_string,MAX_PATHNAME,"%i",gui_prefs.GUIPosY);
	fputs ("\r\nGUIPosY=",gxmame_ini_file);
	fputs (option_string,gxmame_ini_file);
	
	gdk_window_get_size(MainWindow->window, &gui_prefs.GUIWidth,&gui_prefs.GUIHeight);
	
	g_snprintf(option_string,MAX_PATHNAME,"%i",gui_prefs.GUIWidth);
	fputs ("\r\nGUIWidth=",gxmame_ini_file);
	fputs (option_string,gxmame_ini_file);

	g_snprintf(option_string,MAX_PATHNAME,"%i",gui_prefs.GUIHeight);
	fputs ("\r\nGUIHeight=",gxmame_ini_file);
	fputs (option_string,gxmame_ini_file);
	
	g_snprintf(option_string,MAX_PATHNAME,"%i",gui_prefs.current_mode);
	fputs ("\r\nListMode=",gxmame_ini_file);
	fputs (option_string,gxmame_ini_file);
	
	g_snprintf(option_string,MAX_PATHNAME,"%i,%i",
			gui_prefs.ShowFolderList?main_gui.scrolled_window_filters->allocation.width:gui_prefs.Splitters[0],
			gui_prefs.ShowScreenShot?main_gui.scrolled_window_games->allocation.width:gui_prefs.Splitters[1]);
	fputs ("\r\nSplitters=",gxmame_ini_file);
	fputs (option_string,gxmame_ini_file);
	fputs ("\r\n\r\n",gxmame_ini_file);

	fclose(gxmame_ini_file);
	g_free(filename);
	g_free(option_string);
	return TRUE;
}


gboolean save_gxmamerc(void)
{
	FILE *gamerc_file;
	gchar *filename;
	
	filename = g_strdup_printf("%s/.gxmame/gxmamerc",getenv("HOME"));	
	gamerc_file = fopen(filename, "w");
	if (gamerc_file == NULL)
	{
		gxmame_debug("unable to write gxmamerc");
		g_free(filename);
		return (FALSE);
	}
	fputs ("# master config file of GXMame\n",gamerc_file);
	fputs ("mame_executable=",gamerc_file);
	fputs (mame_executable,gamerc_file);
	fputs ("\ncatverDirectory=",gamerc_file);
	fputs (gui_prefs.catverDirectory,gamerc_file);
	fputs ("\nCheatFile=",gamerc_file);
	fputs (gui_prefs.CheatFile,gamerc_file);
	fputs ("\nHiscoreFile=",gamerc_file);
	fputs (gui_prefs.HiscoreFile,gamerc_file);
	fputs ("\nHistoryFile=",gamerc_file);
	fputs (gui_prefs.HistoryFile,gamerc_file);
	fputs ("\nMameInfoFile=",gamerc_file);
	fputs (gui_prefs.MameInfoFile,gamerc_file);
	fputs ("\n",gamerc_file);
	
	fclose(gamerc_file);
	g_free(filename);
	
	return TRUE;
}


gboolean save_dirs_ini(void)
{
	FILE *dir_ini_file = NULL;
	gchar *filename;
	int i;
	
	filename = g_strdup_printf("%s/.gxmame/dirs.ini",getenv("HOME"));	
	
	if ((dir_ini_file = fopen(filename, "w")) == NULL)
	{
		gxmame_debug("unable to write dirs.ini");
		g_free(filename);
		return (FALSE);
	}
	
      
	fputs ("[directories]",dir_ini_file);
	fputs ("\r\nRomPath=",dir_ini_file);
	for (i=0;gui_prefs.RomPath[i]!=NULL;i++)
	{	
		fputs (gui_prefs.RomPath[i],dir_ini_file);
		if (gui_prefs.RomPath[i+1]!=NULL)
			fputs (";",dir_ini_file);
	}
	fputs ("\r\nSamplePath=",dir_ini_file);
	for (i=0;gui_prefs.SamplePath[i]!=NULL;i++)
	{
		fputs (gui_prefs.SamplePath[i],dir_ini_file);
		if (gui_prefs.SamplePath[i+1]!=NULL)
			fputs (";",dir_ini_file);
	}
	fputs ("\r\nInputDirectory=",dir_ini_file);
	fputs (gui_prefs.InputDirectory,dir_ini_file);
	fputs ("\r\nHiscoreDirectory=",dir_ini_file);
	fputs (gui_prefs.HiscoreDirectory,dir_ini_file);
	fputs ("\r\nArtworkDirectory=",dir_ini_file);
	fputs (gui_prefs.ArtworkDirectory,dir_ini_file);
	fputs ("\r\nSnapshotDirectory=",dir_ini_file);
	fputs (gui_prefs.SnapshotDirectory,dir_ini_file);
	
	fputs ("\r\n\r\n",dir_ini_file);
	fclose(dir_ini_file);
	g_free(filename);
	return TRUE;
}


void quick_check(void)
{
	static gboolean quick_check_running;
	GList *list_pointer;
	GList *list_pointer2;
	gchar *filename;
	gint nb_rom_not_checked;
	gfloat done;
	int i;
	struct stat filestatbuf;
	RomEntry *rom;
if(!quick_check_running)
{	/*prevent user to launch several quick check at the same time */
	show_progress_bar();
	quick_check_running=1;
	filename = g_malloc(200);
	nb_rom_not_checked = g_list_length(not_checked_list);
	g_message(_("Performing quick check, please wait:"));
	/* need to use the last or I can find the list anymore*/
	for (list_pointer = g_list_last(not_checked_list);(list_pointer != NULL);)
	{
		rom = (RomEntry *)list_pointer->data;
		/* Looking for roms */
		rom->has_roms = 2;
		for (i=0;gui_prefs.RomPath[i]!=NULL;i++)
		{	
			g_snprintf(filename, 200, "%s%s.zip",gui_prefs.RomPath[i],rom->romname);
		        if(!access(filename,R_OK))
		        {
//		        	printf("rom %s found,%s\n",rom->romname,rom->gamename);
		        	rom->has_roms = 1;
		        }
		        else
		        {	/*test of directories*/
				g_snprintf(filename, 200,"%s%s",gui_prefs.RomPath[i],rom->romname);
				if (stat(filename,&filestatbuf)>=0) 
				{
					rom->has_roms = 1;
				}
				else
				{
		        		if(strcmp(rom->cloneof,"-"))
		        		{
//		        			printf("rom %s not found, trying clone %s\n",rom->romname,rom->cloneof);
		        			g_snprintf(filename, 200, "%s%s.zip",gui_prefs.RomPath[i],rom->cloneof);
		        			if(!access(filename,R_OK))
		        			{
//		        				printf("    clone %s found\n",rom->cloneof);
		        				rom->has_roms = 1;
		        			}
						else
						{
							g_snprintf(filename, 200,"%s%s",gui_prefs.RomPath[i],rom->cloneof);
							if (stat(filename,&filestatbuf)>=0)
							{
								rom->has_roms = 1;
							}
						}
		        		}
				}
		        }
			/* prevent to look into other folders if we already find the rom*/
			if(rom->has_roms == 1) break;
		}
		if(rom->has_roms == 2)
			rom->has_roms = 0;		
		/* Looking for samples */
		if (rom->nb_samples==0)
		{	/* no sample -> no need to check anything */
			rom->has_samples = 0;
		}
		else
		{
			rom->has_samples = 2;
			for (i=0;gui_prefs.SamplePath[i]!=NULL;i++)
			{
				g_snprintf(filename, 200, "%s%s.zip",gui_prefs.SamplePath[i],rom->romname);
			        if(!access(filename,R_OK))
			        {
//			        	printf("samples %s found,%s\n",rom->romname,rom->gamename);
			        	rom->has_samples = 1;
			        }
			        else
			        {	/*test of directories*/
					g_snprintf(filename, 200,"%s%s",gui_prefs.SamplePath[i],rom->romname);
					if (stat(filename,&filestatbuf)>=0)
					{
						rom->has_samples = 1;
					}
					else
					{
			        		if(strcmp(rom->sampleof,"-"))
			        		{
//			        			printf("samples %s not found, trying clone %s\n",rom->romname,rom->sampleof);
			        			g_snprintf(filename, 200, "%s%s.zip",gui_prefs.SamplePath[i],rom->sampleof);
			        			if(!access(filename,R_OK))
			        			{
//			        				printf("    clone %s found\n",rom->sampleof);
			        				rom->has_samples = 1;
			        			}
							else
							{
								g_snprintf(filename, 200,"%s%s",gui_prefs.SamplePath[i],rom->sampleof);
								if (stat(filename,&filestatbuf)>=0)
								{
									rom->has_samples = 1;
								}
							}
			        		}
					}
			        }
				/* prevent to look into other folders if we already find the sample*/
				if(rom->has_samples == 1) break;
			}
			if(rom->has_samples == 2)
				rom->has_samples = 0;
		}
		list_pointer2=list_pointer;
		list_pointer = g_list_previous(list_pointer);
		/* is this remove the list of memory or keep a copy of pointer ?*/
		if (g_list_length(not_checked_list)>1)
			g_list_remove_link(not_checked_list,list_pointer2);
		else{/* I dont like this way to empty/free? the list*/
			list_pointer2->next=NULL;
			list_pointer2->prev=NULL;
			not_checked_list=NULL;
		}
		done = (gfloat)(supported_games - g_list_length(not_checked_list))/(gfloat)supported_games;
//		g_message("\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b\b%6.2f%%",done * 100 );
		update_progress_bar(done);		
		/* allow to frefresh the display during the quick check */
		while(gtk_events_pending())
			gtk_main_iteration();
		/* if the game is in the list, update it */
		if (rom->position > -1)
			update_game_in_list(rom->position);
	}
	g_free(filename);
	quick_check_running=0;
	hide_progress_bar();
}
else gxmame_debug("Quick check already running");
}
